@@ -5,7 +5,7 @@
 Script for running over different parameter combinations. 
 """
 
-# Copyright (C) 2011 University of Zurich. All rights reserved.
+# Copyright (C) 2011  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@ Script for running over different parameter combinations.
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
-__version__ = '$Revision$'
 __author__ = 'Riccardo Murri <riccardo.murri@uzh.ch>, Benjamin Jonen <benjamin.jonen@bf.uzh.ch>'
 # summary of user-visible changes
 __changelog__ = """
@@ -30,6 +29,7 @@ __changelog__ = """
 __docformat__ = 'reStructuredText'
 
 
+from __future__ import absolute_import, print_function
 import gc3libs.debug
 import re, os
 import numpy as np
@@ -192,7 +192,7 @@ class paraLoop(object):
             for ixVar in range(0, len(variables)):
                 self.logger.debug('variable #%d is %s' % (ixVar, variables[ixVar]))
 
-            runDescription = os.path.basename(path_to_para_loop)[:-5]
+            runDescription = 'para'# os.path.basename(path_to_para_loop)[:-5]
             substs = gc3libs.utils.defaultdict(list)
             for ixVar, var in enumerate(variables):
                 self.logger.debug('variable: %s' % variables[ixVar])
