@@ -2,8 +2,9 @@
 #
 """
 """
-#
-__version__ = '$Revision$'
+
+from __future__ import absolute_import, print_function
+
 __author__ = 'Sergio Maffioletti <sergio.maffioletti@gc3.uzh.ch>'
 # summary of user-visible changes
 __changelog__ = """
@@ -14,7 +15,7 @@ __docformat__ = 'reStructuredText'
 
 
 # run script, but allow GC3Pie persistence module to access classes defined here;
-# for details, see: http://code.google.com/p/gc3pie/issues/detail?id=95
+# for details, see: https://github.com/uzh/gc3pie/issues/95
 if __name__ == '__main__':
     import cchem_gfit_abc_workflow
     cchem_gfit_abc_workflow.ABC_Workflow().run()
