@@ -4,7 +4,7 @@
 Driver script for running 'Value Function Iteration' programs
 on the SMSCG infrastructure.
 """
-# Copyright (C) 2011-2012 GC3, University of Zurich. All rights reserved.
+# Copyright (C) 2011-2012  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -20,7 +20,6 @@ on the SMSCG infrastructure.
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
-__version__ = '$Revision$'
 __author__ = 'Riccardo Murri <riccardo.murri@uzh.ch>'
 # summary of user-visible changes
 __changelog__ = """
@@ -32,8 +31,9 @@ __docformat__ = 'reStructuredText'
 
 
 # run script, but allow GC3Pie persistence module to access classes defined here;
-# for details, see: http://code.google.com/p/gc3pie/issues/detail?id=95
+# for details, see: https://github.com/uzh/gc3pie/issues/95
 if __name__ == '__main__':
+from __future__ import absolute_import, print_function
     import george
     george.GeorgeScript().run()
 
@@ -111,14 +111,11 @@ class ValueFunctionIteration(SequentialTaskCollection):
         self.datadir = output_dir
         self.extra = extra_args
 
-        # count initial values
+        # count initial values (i.e. number of lines in file)
+        self.num_input_values = 0
         if os.path.exists(initial_values_file):
-            f = open(initial_values_file, 'r')
-            self.num_input_values = gc3libs.utils.count(f, lambda _: True)
-            f.close()
-        else:
-            # XXX: there's no good default value!
-            self.num_input_values = 0
+            with open(initial_values_file, 'r') as fo:
+                self.num_input_values = sum(1 for i in fo)
 
         # this little piece of black magic is to ensure intermediate
         # filenames appear numerically sorted in `ls -l` output
