@@ -4,7 +4,7 @@
 Driver script for running Turbomole basis benchmarks
 on the SMSCG infrastructure.
 """
-# Copyright (C) 2011-2012 GC3, University of Zurich. All rights reserved.
+# Copyright (C) 2011-2012  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -20,7 +20,9 @@ on the SMSCG infrastructure.
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
-__version__ = '$Revision$'
+
+from __future__ import absolute_import, print_function
+
 __author__ = 'Riccardo Murri <riccardo.murri@uzh.ch>'
 # summary of user-visible changes
 __changelog__ = """
@@ -36,7 +38,7 @@ __changelog__ = """
 __docformat__ = 'reStructuredText'
 
 
-# workaround Issue 95, see: http://code.google.com/p/gc3pie/issues/detail?id=95
+# workaround Issue 95, see: https://github.com/uzh/gc3pie/issues/95
 if __name__ == "__main__":
     import gricomp
     gricomp.GRICompScript().run()
@@ -396,7 +398,7 @@ class BasisSweepPasses(StagedTaskCollection):
         gc3libs.utils.mkdir(self.work_dir)
         ridft_dir = os.path.join(self.work_dir, 'ridft')
         gc3libs.utils.mkdir(ridft_dir)
-        gc3libs.utils.copyfile(self.coord, os.path.join(ridft_dir, 'coord'))
+        shutil.copyfile(self.coord, os.path.join(ridft_dir, 'coord'))
         ridft_define_in = _make_define_in(ridft_dir, self.ridft_in)
         ridft_output_dir =  os.path.join(ridft_dir, 'output')
         # application to run in pass 1
@@ -443,7 +445,7 @@ class BasisSweepPasses(StagedTaskCollection):
             ricc2_dir = os.path.join(self.work_dir,
                                      'cbas-%s/cabs-%s/ricc2' % (cbas, cabs))
             gc3libs.utils.mkdir(ricc2_dir)
-            gc3libs.utils.copyfile(ridft_coord, ricc2_dir)
+            shutil.copyfile(ridft_coord, ricc2_dir)
             ricc2_define_in = _make_define_in(ricc2_dir, ricc2_in)
             ricc2_output_dir = os.path.join(ricc2_dir, 'output')
             # guess duration of the RICC2 job
