@@ -1,5 +1,6 @@
 #! /usr/bin/env python
 
+# Copyright (C) 2021       Google LLC. All rights reserved.
 # Copyright (C) 2009-2019  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -37,10 +38,9 @@ from past.utils import old_div
 from builtins import object
 __docformat__ = 'reStructuredText'
 
-__version__ = '2.6.6'
+__version__ = '2.6.7'
 
 
-import inspect
 import os
 import os.path
 import platform
@@ -67,6 +67,7 @@ import gc3libs.defaults
 from gc3libs.quantity import MB, hours, minutes, seconds, MiB
 from gc3libs.events import TaskStateChange, TermStatusChange
 from gc3libs.compat._collections import OrderedDict
+from gc3libs.compat._inspect import getargspec
 
 import gc3libs.exceptions
 from gc3libs.persistence import Persistable
@@ -85,7 +86,7 @@ it stands for 'fetch the whole contents of the remote
 directory'.
 """
 
-# for compatibility with GC3Pie <2.6.6
+# for compatibility with GC3Pie <2.6.7
 Default = gc3libs.defaults
 
 
@@ -2137,12 +2138,12 @@ class Run(Struct):
 
 def _split_specific_args(fn, argdict):
     """
-    Pop any key appears as an argument name in the definition of
+    Pop keys from `argdict` which appear as argument names in the definition of
     function `fn` out into a separate dictionary, and return it.
 
     *Note:* Argument `argdict` is modified in-place!
     """
-    args, varargs, _, _ = inspect.getargspec(fn)
+    args, varargs, _, _ = getargspec(fn)
     specific_args = {}
     for n, argname in enumerate(args):
         if n == 0 and argname == 'self':
@@ -2155,14 +2156,14 @@ def _split_specific_args(fn, argdict):
     return specific_args
 
 
-def create_core(*conf_files, **extra_args):
+def create_core(*conf_files, cfg_dict=None, **extra_args):
     """Make and return a `gc3libs.core.Core`:class: instance.
 
-    It accepts an optional list of configuration filenames.  Filenames
-    containing a `~` or an environment variable reference, will be
-    expanded automatically. If called without arguments, the paths
-    specified in `gc3libs.defaults.CONFIG_FILE_LOCATIONS` will be
-    used.
+    It accepts an optional list of configuration filenames and a dictionary
+    to create a configuration object from. Filenames containing a `~` or
+    an environment variable reference, will be expanded automatically.
+    If called without arguments, the paths specified in
+    `gc3libs.defaults.CONFIG_FILE_LOCATIONS` will be used.
 
     Any keyword argument matching the name of a parameter used by
     `Core.__init__` is passed to it.  Any leftover keyword argument is
@@ -2185,19 +2186,19 @@ def create_core(*conf_files, **extra_args):
         extra_args['auto_enable_auth'] = True
 
     # make 'em all
-    cfg = Configuration(*conf_files, **extra_args)
+    cfg = Configuration(*conf_files, cfg_dict=cfg_dict, **extra_args)
     return Core(cfg, **core_specific_args)
 
 
-def create_engine(*conf_files, **extra_args):
+def create_engine(*conf_files, cfg_dict=None, **extra_args):
     """
     Make and return a `gc3libs.core.Engine`:class: instance.
 
-    It accepts an optional list of configuration filenames.  Filenames
-    containing a `~` or an environment variable reference, will be
-    expanded automatically. If called without arguments, the paths
-    specified in `gc3libs.Default.CONFIG_FILE_LOCATIONS` will be
-    used.
+    It accepts an optional list of configuration filenames and a dictionary
+    to create a configuration object from. Filenames containing a `~` or
+    an environment variable reference, will be expanded automatically.
+    If called without arguments, the paths specified in
+    `gc3libs.Default.CONFIG_FILE_LOCATIONS` will be used.
 
     Any keyword argument that matches the name of a parameter of the
     constructor for :class:`Engine` is passed to that constructor.
@@ -2211,7 +2212,7 @@ def create_engine(*conf_files, **extra_args):
     # extract `Engine`-specific construction params
     engine_specific_args = _split_specific_args(Engine.__init__, extra_args)
 
-    core = create_core(*conf_files, **extra_args)
+    core = create_core(*conf_files, cfg_dict=cfg_dict, **extra_args)
     return Engine(core, **engine_specific_args)
 
 
