@@ -1,8 +1,11 @@
 #! /usr/bin/env python
 """
-Support for running a generic application with the GC3Libs.
+Specialized support for popular scientific applications.
+
+Each application-specific class is packaged in a separate
+submodule, which see for details.
 """
-# Copyright (C) 2009-2012 GC3, University of Zurich. All rights reserved.
+# Copyright (C) 2009-2012  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -15,14 +18,13 @@ Support for running a generic application with the GC3Libs.
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
+from __future__ import absolute_import, print_function, unicode_literals
 __docformat__ = 'reStructuredText'
-__version__ = '2.2.3 version (SVN $Revision: 4001 $)'
 
 
-## main: run tests
+# main: run tests
 
 if "__main__" == __name__:
     import doctest
