@@ -1,8 +1,9 @@
 #! /usr/bin/env python
-#
+
 """
 """
-# Copyright (C) 2012-2014, GC3, University of Zurich. All rights reserved.
+
+# Copyright (C) 2012-2019  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -15,62 +16,77 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-#
-__docformat__ = 'reStructuredText'
-__version__ = '$Revision$'
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import absolute_import, print_function, unicode_literals
+
+from builtins import str
 
 import hashlib
 import os
 import re
 import paramiko
-import time
 
 # EC2 APIs
 try:
     import boto
     import boto.ec2.regioninfo
     import boto.exception
-except ImportError:
+except ImportError as err:
     from gc3libs.exceptions import ConfigurationError
     raise ConfigurationError(
-        "EC2 backend has been requested but no `boto` package"
-        " was found. Please, install `boto` with `pip install boto`"
-        " or `easy_install boto` and try again, or update your"
-        " configuration file.")
-import Crypto
+        "The EC2 backend is used but the `boto` module"
+        " cannot be used: {err}. Please, either install it with"
+        "`pip install boto` and verify that it works"
+        " by running `python -c 'import boto'`,"
+        " then try again, or update your configuration file and"
+        " disable any EC2-based resources."
+        .format(err=err))
+
+import Crypto.PublicKey.DSA
+import Crypto.PublicKey.RSA
 
 # GC3Pie imports
 import gc3libs
-from gc3libs.exceptions import RecoverableError, UnrecoverableError, \
-    ConfigurationError, LRMSSkipSubmissionToNextIteration, \
-    MaximumCapacityReached, UnrecoverableAuthError, TransportError
+import gc3libs.defaults
+from gc3libs.exceptions import UnrecoverableError, \
+    ConfigurationError, MaximumCapacityReached, ResourceNotReady, \
+    UnrecoverableAuthError, TransportError
 import gc3libs.url
 from gc3libs import Run
-from gc3libs.utils import mkdir, same_docstring_as, insert_char_every_n_chars
+from gc3libs.utils import same_docstring_as, insert_char_every_n_chars
 from gc3libs.backends import LRMS
+from gc3libs.backends.shellcmd import ShellcmdLrms
 from gc3libs.backends.vmpool import VMPool, InstanceNotFound
-from gc3libs.session import Session
-from gc3libs.persistence import Persistable
 
-available_subresource_types = [gc3libs.Default.SHELLCMD_LRMS]
+
+## module metadata
+__docformat__ = 'reStructuredText'
+
+
+## module-level constants
+available_subresource_types = [gc3libs.defaults.SHELLCMD_LRMS]
 
 # example Boto error message:
-#     <Response><Errors><Error><Code>TooManyInstances</Code><Message>Quota exceeded for ram: Requested 8000, but already used 16000 of 16384 ram</Message></Error></Errors><RequestID>req-c219213b-88d2-42dc-a3ab-10ac80aa7df7</RequestID></Response>
+# <Response><Errors><Error><Code>TooManyInstances</Code><Message>Quota exceeded for ram: Requested 8000, but already used 16000 of 16384 ram</Message></Error></Errors><RequestID>req-c219213b-88d2-42dc-a3ab-10ac80aa7df7</RequestID></Response>  # noqa
 #
-_BOTO_ERRMSG_RE = re.compile(r'<Code>(?P<code>[A-Za-z0-9]+)</Code><Message>(?P<message>.*)</Message>', re.X)
+_BOTO_ERRMSG_RE = re.compile(r'<Code>(?P<code>[A-Za-z0-9]+)</Code>'
+                             '<Message>(?P<message>.*)</Message>', re.X)
+
+
+## code
 
 class EC2VMPool(VMPool):
+
     """
     Specific implementation of VMPool
     """
+
     def _get_instance(self, vm_id):
         # contact EC2 API to get VM info
         try:
             reservations = self.conn.get_all_instances(instance_ids=[vm_id])
-        except boto.exception.EC2ResponseError, err:
+        except boto.exception.EC2ResponseError as err:
             # scrape actual error kind and message out of the
             # exception; we do this mostly for sensible logging, but
             # could be an actual improvement to Boto to provide
@@ -102,6 +118,7 @@ class EC2VMPool(VMPool):
 
 
 class EC2Lrms(LRMS):
+
     """
     EC2 resource.
     """
@@ -112,7 +129,7 @@ class EC2Lrms(LRMS):
                  architecture, max_cores, max_cores_per_job,
                  max_memory_per_core, max_walltime,
                  # these are specific of the EC2Lrms class
-                 ec2_region, keypair_name, public_key,
+                 ec2_region, keypair_name, public_key, vm_auth,
                  image_id=None, image_name=None, ec2_url=None,
                  instance_type=None, auth=None, vm_pool_max_size=None,
                  user_data=None, **extra_args):
@@ -140,7 +157,7 @@ class EC2Lrms(LRMS):
 
         self.region = ec2_region
 
-        # Mapping of job.ec2_instance_id => LRMS
+        # Mapping of job.execution._lrms_vm_id => LRMS
         self.subresources = {}
 
         auth = self._auth_fn()
@@ -161,7 +178,8 @@ class EC2Lrms(LRMS):
                 "Keypair name `%s` is invalid: keypair names can only contain "
                 "alphanumeric chars: [a-zA-Z0-9_]" % keypair_name)
         self.keypair_name = keypair_name
-        self.public_key = os.path.expanduser(os.path.expandvars(public_key.strip()))
+        self.public_key = os.path.expanduser(
+            os.path.expandvars(public_key.strip()))
         self.image_id = image_id
         self.image_name = image_name
         self.instance_type = instance_type
@@ -177,12 +195,22 @@ class EC2Lrms(LRMS):
         self.subresource_args['architecture'] = self['architecture']
         self.subresource_args['max_cores'] = self['max_cores']
         self.subresource_args['max_cores_per_job'] = self['max_cores_per_job']
-        self.subresource_args['max_memory_per_core'] = self['max_memory_per_core']
+        self.subresource_args['max_memory_per_core'] = \
+            self['max_memory_per_core']
         self.subresource_args['max_walltime'] = self['max_walltime']
+        # SSH-specific configuration
+        self.subresource_args['transport'] = 'ssh'
+        self.subresource_args['auth'] = vm_auth
+        self.subresource_args['ssh_timeout'] = 7  # FIXME: hard-coded!
+        self.subresource_args['ignore_ssh_host_keys'] = True
+        self.subresource_args['keyfile'] = self.public_key
+        if self.subresource_args['keyfile'].endswith('.pub'):
+            keyfile = self.subresource_args['keyfile'][:-len('.pub')]
+            self.subresource_args['keyfile'] = keyfile
         # ShellcmdLrms by default trusts the configuration, instead of
         # checking the real amount of memory and number of cpus, but
         # we need the real values instead.
-        if self.subresource_type == gc3libs.Default.SHELLCMD_LRMS:
+        if self.subresource_type == gc3libs.defaults.SHELLCMD_LRMS:
             self.subresource_args['override'] = 'True'
 
         if not image_name and not image_id:
@@ -190,12 +218,6 @@ class EC2Lrms(LRMS):
                 "No `image_id` or `image_name` has been specified in the"
                 " configuration file.")
 
-        # helper for creating sub-resources
-        self._cfgobj = gc3libs.config.Configuration(
-            *gc3libs.Default.CONFIG_FILE_LOCATIONS,
-            auto_enable_auth=True)
-
-
     def _connect(self):
         """
         Connect to the EC2 endpoint and check that the required
@@ -205,7 +227,7 @@ class EC2Lrms(LRMS):
             return
 
         args = {
-            'aws_access_key_id':     self.ec2_access_key,
+            'aws_access_key_id': self.ec2_access_key,
             'aws_secret_access_key': self.ec2_secret_key,
         }
 
@@ -272,7 +294,7 @@ class EC2Lrms(LRMS):
         self._connect()
 
         args = {
-            'key_name':  self.keypair_name,
+            'key_name': self.keypair_name,
             'min_count': 1,
             'max_count': 1
         }
@@ -304,7 +326,7 @@ class EC2Lrms(LRMS):
         gc3libs.log.debug("Create new VM using image id `%s`", image_id)
         try:
             reservation = self._conn.run_instances(image_id, **args)
-        except boto.exception.EC2ResponseError, err:
+        except boto.exception.EC2ResponseError as err:
             # scrape actual error kind and message out of the
             # exception; we do this mostly for sensible logging, but
             # could be an actual improvement to Boto to provide
@@ -319,7 +341,7 @@ class EC2Lrms(LRMS):
             else:
                 # fall back to normal reporting...
                 raise UnrecoverableError("Error starting instance: %s" % err)
-        except Exception, ex:
+        except Exception as ex:
             raise UnrecoverableError("Error starting instance: %s" % ex)
         vm = reservation.instances[0]
         self._vmpool.add_vm(vm)
@@ -359,23 +381,11 @@ class EC2Lrms(LRMS):
         because that's the way the fingerprint is returned from the
         EC2 API.
         """
-        return str.join(':', (i.encode('hex') for i in privkey.get_fingerprint()))
+        return ':'.join((i.encode('hex') for i in privkey.get_fingerprint()))
 
     @staticmethod
-    def __pubkey_ssh_fingerprint(privkey):
-        """
-        Compute SSH public key fingerprint like `ssh-keygen -l -f`.
-
-        Return a string representation of the key fingerprint in
-        colon-separated hex format (just like OpenSSH commands print
-        it to the terminal).
-
-        Argument `privkey` is a `paramiko.pkey.PKey` object.
-        """
-        return str.join(':', (i.encode('hex') for i in privkey.get_fingerprint()))
-
-    @staticmethod
-    def __pubkey_aws_fingerprint(privkeypath, reader=Crypto.PublicKey.RSA.importKey):
+    def __pubkey_aws_fingerprint(privkeypath,
+                                 reader=Crypto.PublicKey.RSA.importKey):
         """
         Compute SSH public key fingerprint like AWS EC2 does.
 
@@ -422,11 +432,13 @@ class EC2Lrms(LRMS):
                 self.public_key)
 
         privkey = None
-        local_fingerprints = [ ]
+        local_fingerprints = []
         for format, privkey_reader, pubkey_reader in [
-                ('DSS', paramiko.DSSKey.from_private_key_file, Crypto.PublicKey.DSA.importKey),
-                ('RSA', paramiko.RSAKey.from_private_key_file, Crypto.PublicKey.RSA.importKey),
-                ]:
+                ('DSS', paramiko.DSSKey.from_private_key_file,
+                 Crypto.PublicKey.DSA.importKey),
+                ('RSA', paramiko.RSAKey.from_private_key_file,
+                 Crypto.PublicKey.RSA.importKey),
+        ]:
             try:
                 gc3libs.log.debug(
                     "Trying to load key file `%s` as SSH %s key...",
@@ -435,25 +447,28 @@ class EC2Lrms(LRMS):
                 gc3libs.log.info(
                     "Successfully loaded key file `%s` as SSH %s key.",
                     keyfile, format)
-                # compute public key fingerprints, for comparing them with the remote one
+                # compute public key fingerprints, for comparing them
+                # with the remote one
                 localkey_fingerprints = [
-                    # Usual SSH key fingerprint, computed like `ssh-keygen -l -f`
-                    # This is used, e.g., by OpenStack's EC2 compatibility layer.
+                    # Usual SSH key fingerprint, computed like
+                    # `ssh-keygen -l -f` This is used, e.g., by
+                    # OpenStack's EC2 compatibility layer.
                     self.__pubkey_ssh_fingerprint(privkey),
                     # Amazon EC2 computes the key fingerprint in a
                     # different way, see http://blog.jbrowne.com/?p=23 and
                     # https://gist.github.com/jtriley/7270594 for details.
                     self.__pubkey_aws_fingerprint(keyfile, pubkey_reader),
                 ]
-            ## PasswordRequiredException < SSHException so we must check this first
+            # PasswordRequiredException < SSHException so we must
+            # check this first
             except paramiko.PasswordRequiredException:
                 gc3libs.log.warning(
-                    "Key %s is encripted with a password, so we cannot check if it"
-                    " matches the remote keypair (maybe you should start `ssh-agent`?)."
-                    " Continuing without consistency check with remote fingerprint.",
-                    keyfile)
+                    "Key %s is encripted with a password, so we cannot check "
+                    " if it matches the remote keypair (maybe you should"
+                    " start `ssh-agent`?). Continuing without consistency"
+                    " check with remote fingerprint.", keyfile)
                 return False
-            except paramiko.SSHException, ex:
+            except paramiko.SSHException as ex:
                 gc3libs.log.debug(
                     "File `%s` is not a valid %s private key: %s",
                     keyfile, format, ex)
@@ -464,14 +479,16 @@ class EC2Lrms(LRMS):
                              " RSA key nor a DSS key" % self.public_key)
 
         # check key fingerprint
-        if local_fingerprints and ec2_key.fingerprint not in localkey_fingerprints:
+        if (local_fingerprints
+                and ec2_key.fingerprint not in localkey_fingerprints):
             raise UnrecoverableAuthError(
-                "Keypair `%s` exists but has different fingerprint than local one:"
-                " local public key file `%s` has fingerprint(s) `%s`,"
-                " whereas EC2 API reports fingerprint `%s`. Aborting!" % (
+                "Keypair `%s` exists but has different fingerprint than"
+                " local one: local public key file `%s` has fingerprint(s)"
+                " `%s`, whereas EC2 API reports fingerprint `%s`."
+                " Aborting!" % (
                     self.public_key,
                     self.keypair_name,
-                    str.join('/', localkey_fingerprints),
+                    '/'.join(localkey_fingerprints),
                     ec2_key.fingerprint,
                 ),
                 do_log=True)
@@ -489,8 +506,9 @@ class EC2Lrms(LRMS):
                 gc3libs.log.info(
                     "Successfully imported key `%s`"
                     " with fingerprint `%s` as keypair `%s`",
-                    imported_key.name, imported_key.fingerprint, self.keypair_name)
-            except Exception, ex:
+                    imported_key.name, imported_key.fingerprint,
+                    self.keypair_name)
+            except Exception as ex:
                 raise UnrecoverableError("Error importing keypair %s: %s"
                                          % (self.keypair_name, ex))
 
@@ -500,19 +518,12 @@ class EC2Lrms(LRMS):
         ip using configuration file parameters.
         """
         gc3libs.log.debug(
-            "Creating remote ShellcmdLrms resource for ip %s", remote_ip)
+            "Creating remote ShellcmdLrms resource for IP address %s",
+            remote_ip)
         args = self.subresource_args.copy()
         args['frontend'] = remote_ip
-        args['transport'] = "ssh"
-        args['keyfile'] = self.public_key
-        if args['keyfile'].endswith('.pub'):
-            args['keyfile'] = args['keyfile'][:-4]
-        args['ignore_ssh_host_keys'] = True
         args['name'] = "%s@%s" % (id, self.name)
-        args['auth'] = args['vm_auth']
-        args['ssh_timeout'] = 7
-        resource = self._cfgobj._make_resource(args)
-        return resource
+        return ShellcmdLrms(**args)
 
     def _parse_security_group(self):
         """
@@ -529,9 +540,9 @@ class EC2Lrms(LRMS):
                 continue
             self.security_group_rules.append({
                 'ip_protocol': rulesplit[0],
-                'from_port':   int(rulesplit[1]),
-                'to_port':     int(rulesplit[2]),
-                'cidr_ip':     rulesplit[3],
+                'from_port': int(rulesplit[1]),
+                'to_port': int(rulesplit[2]),
+                'cidr_ip': rulesplit[3],
             })
 
     def _setup_security_groups(self):
@@ -553,7 +564,7 @@ class EC2Lrms(LRMS):
                 security_group = self._conn.create_security_group(
                     self.security_group_name,
                     "GC3Pie_%s" % self.security_group_name)
-            except Exception, ex:
+            except Exception as ex:
                 gc3libs.log.error("Error creating security group %s: %s",
                                   self.security_group_name, ex)
                 raise UnrecoverableError(
@@ -566,21 +577,40 @@ class EC2Lrms(LRMS):
                         "Adding rule %s to security group %s.",
                         rule, self.security_group_name)
                     security_group.authorize(**rule)
-                except Exception, ex:
-                    gc3libs.log.info("Ignoring error adding rule %s to"
-                                     " security group %s: %s", str(rule),
-                                     self.security_group_name, str(ex))
-
+                except Exception as ex:
+                    if gc3libs.error_ignored(
+                            # context:
+                            # - module
+                            'ec2',
+                            # - class
+                            'EC2Lrms',
+                            # - method
+                            'setup_security_groups',
+                            # - actual error class
+                            ex.__class__.__name__,
+                            # - additional keywords
+                            'setup',
+                            'security',
+                            'network',
+                            'cloud',
+                    ):
+                        gc3libs.log.info(
+                            "Ignoring error adding rule %s"
+                            " to security group %s: %s",
+                            rule, self.security_group_name, ex)
+                    else:
+                        # propagate exception to caller
+                        raise
         else:
             # Check if the security group has all the rules we want
             security_group = groups[self.security_group_name]
             current_rules = []
             for rule in security_group.rules:
                 rule_dict = {
-                    'ip_protocol':  rule.ip_protocol,
-                    'from_port':    int(rule.from_port),
-                    'to_port':      int(rule.to_port),
-                    'cidr_ip':      str(rule.grants[0]),
+                    'ip_protocol': rule.ip_protocol,
+                    'from_port': int(rule.from_port),
+                    'to_port': int(rule.to_port),
+                    'cidr_ip': str(rule.grants[0]),
                 }
                 current_rules.append(rule_dict)
 
@@ -625,7 +655,7 @@ class EC2Lrms(LRMS):
 
     @same_docstring_as(LRMS.cancel_job)
     def cancel_job(self, app):
-        resource = self._get_subresource(self._get_vm(app.ec2_instance_id))
+        resource = self._get_subresource(self._get_vm(app.execution._lrms_vm_id))
         return resource.cancel_job(app)
 
     @same_docstring_as(LRMS.get_resource_status)
@@ -642,7 +672,7 @@ class EC2Lrms(LRMS):
         for vm_id in self._vmpool:
             try:
                 vm = self._vmpool.get_vm(vm_id)
-            except UnrecoverableError, ex:
+            except UnrecoverableError as ex:
                 gc3libs.log.warning(
                     "Removing stale information on VM `%s`. It has probably"
                     " been deleted from outside GC3Pie.", vm_id)
@@ -663,48 +693,70 @@ class EC2Lrms(LRMS):
                 self._vmpool.remove_vm(vm.id)
             elif vm.state == 'terminated':
                 gc3libs.log.info(
-                    "VM `%s` in TERMINATED state. It has probably been terminated"
-                    " from outside GC3Pie. Removing it from the list of VM.",
-                    vm.id)
+                    "VM `%s` in TERMINATED state. It has probably been"
+                    " terminated from outside GC3Pie. Removing it from the"
+                    " list of VM.", vm.id)
                 self._vmpool.remove_vm(vm.id)
             elif vm.state in ['shutting-down', 'stopped']:
                 # The VM has probably ben stopped or shut down from
                 # outside GC3Pie.
                 gc3libs.log.error(
-                    "VM with id `%s` is in terminal state `%s`.", vm.id, vm.state)
+                    "VM with id `%s` is in terminal state `%s`.",
+                    vm.id, vm.state)
 
             # Get or create a resource associated to the vm
             resource = self._get_subresource(vm)
             try:
                 resource.get_resource_status()
-            except TransportError, ex:
+            except TransportError as ex:
                 for ip in [vm.public_dns_name, vm.private_ip_address]:
                     if vm.preferred_ip == ip:
                         continue
                     vm.preferred_ip = ip
                     resource.frontend = ip
                     gc3libs.log.info(
-                        "Connection error. Trying with secondary IP address %s",
-                        vm.preferred_ip)
+                        "Connection error. Trying with secondary IP"
+                        " address %s", vm.preferred_ip)
                     try:
                         resource.get_resource_status()
                         break
-                    except Exception, ex:
-                        gc3libs.log.info(
-                            "Ignoring error while updating resource %s. "
-                            "The corresponding VM may not be ready yet. Error: %s",
+                    except Exception as ex:
+                        # XXX: I'm exempting this from the GC3Pie
+                        # `error_ignored()` policy, since this is a kind of
+                        # "expected" error -- it *will* happen if the VM has
+                        # not booted up yet or if we're hunting for the correct
+                        # address.
+                        gc3libs.log.debug(
+                            "Ignoring error in updating resource %s: %s"
+                            " The corresponding VM may not be ready yet.",
                             resource.name, ex)
-            except Exception, ex:
-                # XXX: Actually, we should try to identify the kind of
+            except Exception as ex:
+                # FIXME: Actually, we should try to identify the kind of
                 # error we are getting. For instance, if the
                 # configuration options `username` is wrong, we will
                 # create VMs but we will never be able to submit jobs
                 # to them, thus causing an increasing number of
                 # useless VMs created on the cloud.
-                gc3libs.log.info(
-                    "Ignoring error while updating resource %s. "
-                    "The corresponding VM may not be ready yet. Error: %s",
-                    resource.name, ex)
+                if gc3libs.error_ignored(
+                        # context:
+                        # - module
+                        'ec2',
+                        # - class
+                        'EC2Lrms',
+                        # - method
+                        'get_resource_status',
+                        # - actual error class
+                        ex.__class__.__name__,
+                        # - additional keywords
+                        'vm',
+                ):
+                    gc3libs.log.info(
+                        "Ignoring error in updating resource %s: %s"
+                        "The corresponding VM may not be ready yet.",
+                        resource.name, ex)
+                else:
+                    # propagate exception to caller
+                    raise
             if resource.updated:
                 # Update also the instance_type specs, if not
                 # already updated
@@ -720,34 +772,34 @@ class EC2Lrms(LRMS):
         return self
 
     @same_docstring_as(LRMS.get_results)
-    def get_results(self, app, download_dir, overwrite=False, changed_only=True):
-        subresource = self._get_subresource(self._get_vm(app.ec2_instance_id))
+    def get_results(self, app, download_dir, overwrite=False,
+                    changed_only=True):
+        subresource = self._get_subresource(self._get_vm(app.execution._lrms_vm_id))
         return subresource.get_results(app, download_dir,
-                                    ignore_nonexisting=False,
-                                    overwrite=overwrite,
-                                    changed_only=changed_only)
+                                       overwrite=overwrite,
+                                       changed_only=changed_only)
 
     @same_docstring_as(LRMS.update_job_state)
     def update_job_state(self, app):
-        if app.ec2_instance_id not in self.subresources:
+        if app.execution._lrms_vm_id not in self.subresources:
             try:
-                self.subresources[app.ec2_instance_id] = self._get_subresource(
-                    self._get_vm(app.ec2_instance_id))
-            except InstanceNotFound, ex:
+                self.subresources[app.execution._lrms_vm_id] = self._get_subresource(
+                    self._get_vm(app.execution._lrms_vm_id))
+            except InstanceNotFound as ex:
                 gc3libs.log.error(
                     "Changing state of task '%s' to TERMINATED since EC2 "
                     "instance '%s' does not exist anymore.",
-                    app.execution.lrms_jobid, app.ec2_instance_id)
+                    app.execution.lrms_jobid, app.execution._lrms_vm_id)
                 app.execution.state = Run.State.TERMINATED
                 raise ex
-            except UnrecoverableError, ex:
+            except UnrecoverableError as ex:
                 gc3libs.log.error(
                     "Changing state of task '%s' to UNKNOWN because of "
                     "an EC2 error.", app.execution.lrms_jobid)
                 app.execution.state = Run.State.UNKNOWN
                 raise ex
 
-        return self.subresources[app.ec2_instance_id].update_job_state(app)
+        return self.subresources[app.execution._lrms_vm_id].update_job_state(app)
 
     def submit_job(self, job):
         """
@@ -786,7 +838,7 @@ class EC2Lrms(LRMS):
         # Updating resource is needed to update the subresources. This
         # is not always done before the submit_job because of issue
         # nr.  386:
-        #     http://code.google.com/p/gc3pie/issues/detail?id=386
+        #     https://github.com/uzh/gc3pie/issues/386
         self.get_resource_status()
 
         pending_vms = set(vm.id for vm in self._vmpool.get_all_vms()
@@ -810,7 +862,7 @@ class EC2Lrms(LRMS):
                         self.instance_type, job.jobname))
 
         # First of all, try to submit to one of the subresources.
-        for vm_id, resource in self.subresources.items():
+        for vm_id, resource in list(self.subresources.items()):
             if not resource.updated:
                 # The VM is probably still booting, let's skip to the
                 # next one and add it to the list of "pending" VMs.
@@ -820,19 +872,36 @@ class EC2Lrms(LRMS):
                 # Check that the required image id and instance type
                 # are correct
                 vm = self._get_vm(vm_id)
-                if vm.image_id != image_id or vm.instance_type != instance_type:
+                if (vm.image_id != image_id
+                        or vm.instance_type != instance_type):
                     continue
                 resource.submit_job(job)
-                job.ec2_instance_id = vm_id
+                job.execution._lrms_vm_id = vm_id
                 job.changed = True
                 gc3libs.log.info(
                     "Job successfully submitted to remote resource %s.",
                     resource.name)
                 return job
-            except gc3libs.exceptions.LRMSSubmitError, ex:
-                gc3libs.log.debug(
-                    "Ignoring error while submit to resource %s: %s. ",
-                    resource.name, str(ex))
+            except gc3libs.exceptions.LRMSSubmitError as ex:
+                if gc3libs.error_ignored(
+                        # context:
+                        # - module
+                        'ec2',
+                        # - class
+                        'EC2Lrms',
+                        # - method
+                        'submit_job',
+                        # - actual error class
+                        ex.__class__.__name__,
+                        # - additional keywords
+                        'submit',
+                ):
+                    gc3libs.log.debug(
+                        "Ignoring error in submitting to resource %s: %s. ",
+                        resource.name, ex)
+                else:
+                    # propagate exception to caller
+                    raise
 
         # Couldn't submit to any resource.
         if not pending_vms:
@@ -861,15 +930,14 @@ class EC2Lrms(LRMS):
             "No available resource was found, but some VM is still in"
             " `pending` state. Waiting until the next iteration before"
             " creating a new VM. Pending VM ids: %s", pending_vms)
-        raise LRMSSkipSubmissionToNextIteration(
-            "Delaying submission until some of the VMs currently pending"
-            " is ready. Pending VM ids: %s"
-            % str.join(', ', pending_vms))
+        raise ResourceNotReady(
+            "Delaying submission until one of the VMs currently pending"
+            " is ready. (Pending VM ids: %r)" % (pending_vms,))
 
     @same_docstring_as(LRMS.peek)
     def peek(self, app, remote_filename, local_file, offset=0, size=None):
         resource = self._get_subresource(
-            self._get_vm(app.ec2_instance_id))
+            self._get_vm(app.execution._lrms_vm_id))
         return resource.peek(app, remote_filename, local_file, offset, size)
 
     def validate_data(self, data_file_list=None):
@@ -877,7 +945,7 @@ class EC2Lrms(LRMS):
         Supported protocols: file
         """
         for url in data_file_list:
-            if not url.scheme in ['file']:
+            if url.scheme not in ['file']:
                 return False
         return True
 
@@ -899,16 +967,16 @@ class EC2Lrms(LRMS):
 
         # freeing the resource from the application is now needed as
         # the same instanc may run multiple applications
-        resource = self._get_subresource(self._get_vm(app.ec2_instance_id))
+        resource = self._get_subresource(self._get_vm(app.execution._lrms_vm_id))
         resource.free(app)
 
         # FIXME: current approach in terminating running instances:
         # if no more applications are currently running, turn the instance off
         # check with the associated resource
         resource.get_resource_status()
-        if len(resource.job_infos) == 0:
+        if not resource.has_running_tasks():
             # turn VM off
-            vm = self._get_vm(app.ec2_instance_id)
+            vm = self._get_vm(app.execution._lrms_vm_id)
             gc3libs.log.info("VM instance %s at %s is no longer needed."
                              " Terminating.", vm.id, vm.public_dns_name)
             del self.subresources[vm.id]
@@ -927,20 +995,19 @@ class EC2Lrms(LRMS):
             return
         # Update status of VMs and remote resources
         self.get_resource_status()
-        for vm_id, resource in self.subresources.items():
-            if resource.updated and not resource.job_infos:
+        for vm_id, resource in list(self.subresources.items()):
+            if resource.updated and not resource.has_running_tasks():
                 vm = self._get_vm(vm_id)
                 gc3libs.log.warning(
                     "VM instance %s at %s is no longer needed. "
                     "You may need to terminate it manually.",
                     vm.id, vm.public_dns_name)
-                vm.terminate()
                 del self._vmpool[vm.id]
             resource.close()
         # self._session.save_all()
 
 
-## main: run tests
+# main: run tests
 
 if "__main__" == __name__:
     import doctest
