@@ -1,10 +1,11 @@
 #! /usr/bin/env python
-#
+
 """
 Job control on PBS/Torque clusters (possibly connecting to the
 front-end via SSH).
 """
-# Copyright (C) 2009-2014 GC3, University of Zurich. All rights reserved.
+
+# Copyright (C) 2009-2014, 2016, 2019  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -17,38 +18,29 @@ front-end via SSH).
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-#
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+from __future__ import absolute_import, print_function, unicode_literals
+from builtins import str
 __docformat__ = 'reStructuredText'
-__version__ = '2.2.3 version (SVN $Revision$)'
 
 
 import datetime
-import os
-import posixpath
-import random
 import re
-import sys
-import tempfile
 import time
 
-from gc3libs.compat._collections import defaultdict
-
 from gc3libs import log, Run
 from gc3libs.backends import LRMS
 import gc3libs.exceptions
-from gc3libs.quantity import Memory, kB, MB, GB
-from gc3libs.quantity import Duration, seconds, minutes, hours
-import gc3libs.utils as utils  # first, to_bytes
-from gc3libs.utils import same_docstring_as
+from gc3libs.quantity import Memory
+from gc3libs.quantity import Duration
+from gc3libs.utils import (same_docstring_as, sh_quote_safe_cmdline,
+                           sh_quote_unsafe_cmdline)
 
-import transport
-
-import batch
+from . import batch
 
 
-## data for parsing PBS commands output
+# data for parsing PBS commands output
 
 # regexps for extracting relevant strings
 
@@ -56,28 +48,11 @@ _qsub_jobid_re = re.compile(r'(?P<jobid>\d+).*', re.I)
 
 _qstat_line_re = re.compile(
     r'^(?P<jobid>\d+)[^\d]+\s+'
-    '(?P<jobname>[^\s]+)\s+'
-    '(?P<username>[^\s]+)\s+'
-    '(?P<time_used>[^\s]+)\s+'
-    '(?P<state>[^\s]+)\s+'
-    '(?P<queue>[^\s]+)')
-
-_tracejob_queued_re = re.compile(
-    '(?P<submission_time>\d+/\d+/\d+\s+\d+:\d+:\d+)\s+.\s+'
-    'Job Queued at request of .*job name =\s*(?P<job_name>[^,]+),'
-    '\s+queue =\s*(?P<queue>[^,]+)')
-
-_tracejob_run_re = re.compile(
-    '(?P<running_time>\d+/\d+/\d+\s+\d+:\d+:\d+)\s+.\s+'
-    'Job Run at request of .*')
-
-_tracejob_last_re = re.compile(
-    '(?P<end_time>\d+/\d+/\d+\s+\d+:\d+:\d+)\s+.'
-    '\s+Exit_status=(?P<exit_status>\d+)\s+'
-    'resources_used.cput=(?P<used_cpu_time>[^ ]+)\s+'
-    'resources_used.mem=(?P<mem>[^ ]+)\s+'
-    'resources_used.vmem=(?P<used_memory>[^ ]+)\s+'
-    'resources_used.walltime=(?P<used_walltime>[^ ]+)')
+    r'(?P<jobname>[^\s]+)\s+'
+    r'(?P<username>[^\s]+)\s+'
+    r'(?P<time_used>[^\s]+)\s+'
+    r'(?P<state>[^\s]+)\s+'
+    r'(?P<queue>[^\s]+)')
 
 # convert data to GC3Pie internal format
 
@@ -89,6 +64,7 @@ def _to_memory(val):
 
     Examples::
 
+      >>> from gc3libs.quantity import kB, MB, GB
       >>> _to_memory('44kb') == 44*kB
       True
       >>> _to_memory('12mb') == 12*MB
@@ -103,13 +79,13 @@ def _to_memory(val):
     unit = val[-2:]
     # XXX: check that PBS uses base-2 units
     if unit == 'kb':
-        return int(val[:-2])*Memory.kB
+        return int(val[:-2]) * Memory.kB
     elif unit == 'mb':
-        return int(val[:-2])*Memory.MB
+        return int(val[:-2]) * Memory.MB
     elif unit == 'gb':
-        return int(val[:-2])*Memory.GB
+        return int(val[:-2]) * Memory.GB
     elif unit == 'tb':
-        return int(val[:-2])*Memory.TB
+        return int(val[:-2]) * Memory.TB
     else:
         if val[-1] == 'b':
             # XXX bytes
@@ -117,7 +93,7 @@ def _to_memory(val):
         else:
             # a pure number
             val = int(val)
-        return int(val)*Memory.B
+        return int(val) * Memory.B
 
 
 def _parse_asctime(val):
@@ -125,35 +101,14 @@ def _parse_asctime(val):
         # XXX: replace with datetime.strptime(...) in Python 2.5+
         return datetime.datetime(
             *(time.strptime(val, '%m/%d/%Y %H:%M:%S')[0:6]))
-    except Exception, err:
+    except Exception as err:
         gc3libs.log.error(
             "Cannot parse '%s' as a PBS-format time stamp: %s: %s",
             val, err.__class__.__name__, str(err))
         return None
 
 
-_tracejob_keyval_mapping = {
-    # regexp group name
-    # |               `Task.execution` attribute
-    # |               |
-    # |               |                       converter function
-    # |               |                       |
-    # |               |                       |
-    #   ... common backend attrs (see Issue 78) ...
-    'exit_status': ('exitcode', int),
-    'used_cpu_time': ('used_cpu_time', Duration),
-    'used_walltime': ('duration', Duration),
-    'used_memory': ('max_used_memory', _to_memory),
-    #   ... PBS-only attrs ...
-    'mem': ('pbs_max_used_ram', _to_memory),
-    'submission_time': ('pbs_submission_time', _parse_asctime),
-    'running_time': ('pbs_running_time', _parse_asctime),
-    'end_time': ('pbs_end_time', _parse_asctime),
-    'queue': ('pbs_queue', str),
-    'job_name': ('pbs_jobname', str), }
-
-
-## code
+# code
 
 def count_jobs(qstat_output, whoami):
     """
@@ -174,7 +129,6 @@ def count_jobs(qstat_output, whoami):
     total_queued = 0
     own_running = 0
     own_queued = 0
-    n = 0
     for line in qstat_output.split('\n'):
         log.info("Output line: %s" % line)
         m = _qstat_line_re.match(line)
@@ -194,6 +148,7 @@ def count_jobs(qstat_output, whoami):
 
 
 class PbsLrms(batch.BatchSystem):
+
     """
     Job control on PBS/Torque clusters (possibly by connecting via SSH
     to a submit node).
@@ -210,7 +165,8 @@ class PbsLrms(batch.BatchSystem):
                  frontend, transport,
                  # these are specific to this backend
                  queue=None,
-                 # (Note that optional arguments to the `BatchSystem` class, e.g.:
+                 # (Note that optional arguments to the `BatchSystem` class,
+                 # e.g.:
                  #     keyfile=None, accounting_delay=15,
                  # are collected into `extra_args` and should not be explicitly
                  # spelled out in this signature.)
@@ -240,7 +196,8 @@ class PbsLrms(batch.BatchSystem):
         qsub_argv, app_argv = app.qsub_pbs(self)
         if self.queue is not None:
             qsub_argv += ['-q', ('%s' % self.queue)]
-        return (str.join(' ', qsub_argv), "cd $PBS_O_WORKDIR; " + str.join(' ', app_argv) + '\n')
+        return (sh_quote_safe_cmdline(qsub_argv),
+                'cd "$PBS_O_WORKDIR"; ' + sh_quote_unsafe_cmdline(app_argv))
 
     def _stat_command(self, job):
         return "%s %s | grep ^%s" % (
@@ -252,73 +209,121 @@ class PbsLrms(batch.BatchSystem):
     def _secondary_acct_command(self, job):
         return "%s -x -f %s" % (self._qstat, job.lrms_jobid)
 
-    def _parse_stat_output(self, stdout):
-        # check that passed object obeys contract
-
+    def _parse_stat_output(self, stdout, stderr):
         # parse `qstat` output
-        job_status = stdout.split()[4]
-        jobstatus = dict()
-        log.debug("translating PBS/Torque's `qstat` code "
-                  "'%s' to gc3libs.Run.State", job_status)
-        if job_status in ['Q', 'W']:
-            jobstatus['state'] = Run.State.SUBMITTED
-        elif job_status in ['R']:
-            jobstatus['state'] = Run.State.RUNNING
-        elif job_status in ['S', 'H', 'T'] or 'qh' in job_status:
-            jobstatus['state'] = Run.State.STOPPED
-        elif job_status in ['C', 'E', 'F']:
-            jobstatus['state'] = Run.State.TERMINATING
+        pbs_status = stdout.split()[4]
+        log.debug("translating PBS/Torque's `qstat` code"
+                  " '%s' to gc3libs.Run.State", pbs_status)
+        if pbs_status in ['Q', 'W']:
+            state = Run.State.SUBMITTED
+        elif pbs_status in ['R']:
+            state = Run.State.RUNNING
+        elif pbs_status in ['S', 'H', 'T'] or 'qh' in pbs_status:
+            state = Run.State.STOPPED
+        elif pbs_status in ['C', 'E', 'F']:
+            state = Run.State.TERMINATING
         else:
-            jobstatus['state'] = Run.State.UNKNOWN
+            state = Run.State.UNKNOWN
+        return self._stat_result(state, None)  # no term status info
 
-        return jobstatus
+    _tracejob_queued_re = re.compile(
+        r'(?P<submission_time>\d+/\d+/\d+\s+\d+:\d+:\d+)\s+.\s+'
+        r'Job Queued at request of .*job name =\s*(?P<job_name>[^,]+),'
+        r'\s+queue =\s*(?P<queue>[^,]+)')
 
-    def _parse_acct_output(self, stdout):
-        jobstatus = {}
+    _tracejob_run_re = re.compile(
+        r'(?P<running_time>\d+/\d+/\d+\s+\d+:\d+:\d+)\s+.\s+'
+        r'Job Run at request of .*')
+
+    _tracejob_last_re = re.compile(
+        r'(?P<end_time>\d+/\d+/\d+\s+\d+:\d+:\d+)\s+.'
+        r'\s+Exit_status=(?P<exit_status>\d+)\s+'
+        r'resources_used.cput=(?P<used_cpu_time>[^ ]+)\s+'
+        r'resources_used.mem=(?P<mem>[^ ]+)\s+'
+        r'resources_used.vmem=(?P<used_memory>[^ ]+)\s+'
+        r'resources_used.walltime=(?P<used_walltime>[^ ]+)')
+
+    _tracejob_keyval_mapping = {
+        # regexp group name
+        # |               `Task.execution` attribute
+        # |               |
+        # |               |                        converter function
+        # |               |                        |
+        # |               |                        |
+        #   ... common backend attrs (see Issue 78) ...
+        'exit_status':   ('exitcode',              int),
+        'used_cpu_time': ('used_cpu_time',         Duration),
+        'used_walltime': ('duration',              Duration),
+        'used_memory':   ('max_used_memory',       _to_memory),
+        #   ... PBS-only attrs ...
+        'mem':           ('pbs_max_used_ram',      _to_memory),
+        'submission_time': ('pbs_submission_time', _parse_asctime),
+        'running_time':  ('pbs_running_time',      _parse_asctime),
+        'end_time':      ('pbs_end_time',          _parse_asctime),
+        'queue':         ('pbs_queue',             str),
+        'job_name':      ('pbs_jobname',           str),
+    }
+
+    def _parse_acct_output(self, stdout, stderr):
+        """Parse `tracejob` output."""
+        acctinfo = {}
         for line in stdout.split('\n'):
-            # FIXME: make a list of the three regexps and loop over it
-            match = _tracejob_queued_re.match(line)
-            if match:
-                for key, value in match.groupdict().iteritems():
-                    attr, conv = _tracejob_keyval_mapping[key]
-                    jobstatus[attr] = conv(value)
-                continue
-            match = _tracejob_run_re.match(line)
-            if match:
-                for key, value in match.groupdict().iteritems():
-                    attr, conv = _tracejob_keyval_mapping[key]
-                    jobstatus[attr] = conv(value)
-                continue
-            match = _tracejob_last_re.match(line)
-            if match:
-                for key, value in match.groupdict().iteritems():
-                    attr, conv = _tracejob_keyval_mapping[key]
-                    jobstatus[attr] = conv(value)
-                break
-        return jobstatus
+            for pattern, carry_on in [
+                    # regexp                   exit loop?
+                    # =====================    ==========
+                    (self._tracejob_queued_re, True),
+                    (self._tracejob_run_re,    True),
+                    (self._tracejob_last_re,   False),
+            ]:
+                match = pattern.match(line)
+                if match:
+                    for key, value in match.groupdict().items():
+                        attr, conv = self._tracejob_keyval_mapping[key]
+                        acctinfo[attr] = conv(value)
+                    if carry_on:
+                        continue
+                    else:
+                        break
+        assert 'exitcode' in acctinfo, (
+            "Could not extract exit code from `tracejob` output")
+        acctinfo['termstatus'] = Run.shellexit_to_returncode(
+            acctinfo.pop('exitcode'))
+        return acctinfo
 
-    def _parse_secondary_acct_output(self, stdout):
-        jobstatus = {}
+    _pbspro_keyval_mapping = {
+        # PBS output key
+        # |               `Task.execution` attribute
+        # |                          |                  converter function
+        # |                          |                  |
+        # |                          |                  |
+        #   ... common backend attrs (see Issue 78) ...
+        'Exit_status':             ('exitcode',         int),
+        'resources_used.cpupt':    ('used_cpu_time',    Duration),
+        'resources_used.cput':     ('used_cpu_time',    Duration),
+        'resources_used.vmem':     ('used_memory',      _to_memory),
+        'resources_used.walltime': ('used_walltime',    Duration),
+        #   ... PBS-only attrs ...
+        'etime':                   ('pbs_queued_at',    _parse_asctime),
+        'queue':                   ('pbs_queue',        str),
+        'resources_used.mem':      ('pbs_max_used_ram', _to_memory),
+        'stime':                   ('pbs_started_at',   _parse_asctime),
+    }
+
+    def _parse_secondary_acct_output(self, stdout, stderr):
+        """Parse `qstat -x -f` output (PBSPro only)."""
+        acctinfo = {}
+        # FIXME: could be a bit smarter and not use a dumb quadratic
+        # complexity algo...
         for line in stdout.split('\n'):
-            if 'queue = ' in line:
-                jobstatus['queue'] = line.split('=')[1].strip()
-            if 'Exit_status =' in line:
-                jobstatus['exitcode'] = line.split('=')[1].strip()
-            if 'resources_used.cpupt = ' in line:
-                jobstatus['used_cpu_time'] = line.split('=')[1].strip()
-            if 'resources_used.cput =' in line:
-                jobstatus['used_cpu_time'] = line.split('=')[1].strip()
-            if 'resources_used.mem =' in line:
-                jobstatus['mem'] = line.split('=')[1].strip()
-            if 'resources_used.vmem =' in line:
-                jobstatus['used_memory'] = line.split('=')[1].strip()
-            if 'resources_used.walltime =' in line:
-                jobstatus['used_walltime'] = line.split('=')[1].strip()
-            if 'stime' in line:
-                jobstatus['pbs_started_at'] = line.split('=')[1].strip()
-            if 'etime' in line:
-                jobstatus['pbs_queued_at'] = line.split('=')[1].strip()
-        return jobstatus
+            for key, (attr, conv) in self._pbspro_keyval_mapping:
+                if (key + ' = ') in line:
+                    value = line.split('=')[1].strip()
+                    acctinfo[attr] = conv(value)
+        assert 'exitcode' in acctinfo, (
+            "Could not extract exit code from `qstat -x -f` output")
+        acctinfo['termstatus'] = Run.shellexit_to_returncode(
+            acctinfo.pop('exitcode'))
+        return acctinfo
 
     def _cancel_command(self, jobid):
         return ("%s %s" % (self._qdel, jobid))
@@ -339,7 +344,7 @@ class PbsLrms(batch.BatchSystem):
                 raise gc3libs.exceptions.LRMSError(
                     "PBS backend failed executing '%s':"
                     " exit code: %d; stdout: '%s', stderr: '%s'"
-                    % (_command, exit_code, stdout, stderr))
+                    % (_command, exit_code, qstat_stdout, stderr))
 
             log.debug("Computing updated values for total/available slots ...")
             (total_running, self.queued, self.user_run, self.user_queued) \
@@ -363,7 +368,7 @@ class PbsLrms(batch.BatchSystem):
                      )
             return self
 
-        except Exception, ex:
+        except Exception as ex:
             # self.transport.close()
             log.error("Error querying remote LRMS, see debug log for details.")
             log.debug("Error querying LRMS: %s: %s",
@@ -371,9 +376,8 @@ class PbsLrms(batch.BatchSystem):
             raise
 
 
-## main: run tests
-
+# main: run tests
 if "__main__" == __name__:
     import doctest
-    doctest.testmod(name="sge",
+    doctest.testmod(name="pbs",
                     optionflags=doctest.NORMALIZE_WHITESPACE)
