@@ -1,5 +1,5 @@
 #! /usr/bin/env python
-#
+
 """
 Facade to store and retrieve Job information from permanent storage.
 
@@ -33,7 +33,8 @@ the trick!
   the Python interpreter, or GC3Utils will still complain!
 
 """
-# Copyright (C) 2009-2012 GC3, University of Zurich. All rights reserved.
+
+# Copyright (C) 2009-2012, 2019  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -46,23 +47,25 @@ the trick!
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-#
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+from __future__ import absolute_import, print_function, unicode_literals
 __docformat__ = 'reStructuredText'
-__version__ = '2.2.3 version (SVN $Revision$)'
 
 
 # Export the "public API" towards other modules, so that
 # one can do ``import gc3libs.persistence`` and load whatever
 # should be normally needed and supported.  Other modules in
 # this package should be considered "internal use only".
-from filesystem import FilesystemStore
-from idfactory import IdFactory, JobIdFactory
-from store import make_store, Persistable
+from .filesystem import FilesystemStore
+from .idfactory import IdFactory, JobIdFactory
+from .serialization import Persistable
+from .store import make_store
 
+__all__ = ['make_store', 'Persistable', 'IdFactory',
+           'JobIdFactory', 'FilesystemStore']
 
-## main: run tests
+# main: run tests
 
 if "__main__" == __name__:
     import doctest
