@@ -2,7 +2,7 @@
 #
 """
 """
-# Copyright (C) 2011-2012, GC3, University of Zurich. All rights reserved.
+# Copyright (C) 2011-2019  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -15,19 +15,25 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-__docformat__ = 'reStructuredText'
-__version__ = '$Revision$'
 
+# stdlib imports
+from __future__ import absolute_import, print_function, unicode_literals
+from builtins import str
+from builtins import object
+from abc import ABCMeta, abstractmethod
 
 # GC3Pie imports
 import gc3libs
 from gc3libs.url import Url
+from future.utils import with_metaclass
 
 
-class Store(object):
+__docformat__ = 'reStructuredText'
+
+
+class Store(with_metaclass(ABCMeta, object)):
     """
     Interface for storing and retrieving objects on permanent storage.
 
@@ -46,6 +52,48 @@ class Store(object):
         parts of the code.
     """
 
+    def __init__(self, url=None):
+        if url and not isinstance(url, Url):
+            url = Url(url)
+        self.url = url
+
+    def pre_fork(self):
+        """
+        Make preparations for `fork()`-ing the current process.
+
+        This should close open network connections or any other
+        sockets or file descriptors that cannot be used by both the
+        parent and child process.
+
+        The default implementation of this method does nothing; as of
+        2018-04-10, the only subclass making use of this functionality
+        is `SqlStore`:class:, which needs to dispose the SQLAlchemy
+        engine and re-create it after forking.
+        """
+        pass
+
+    def post_fork(self):
+        """
+        Restore functionality that was suspended in `pre_fork`:meth:
+
+        This method will be called after forking/daemonizing has been
+        successfully accomplished.
+
+        The default implementation of this method does nothing.
+        """
+        pass
+
+    def invalidate_cache(self):
+        """
+        Clear the loaded objects cache (if any).
+
+        Subsequent `load()`:meth: calls are guaranteed to re-load the
+        data directly from the backing store.
+        """
+        # The default implementation of this method does nothing, since
+        # a `Store` subclass may not keep a cache of loaded objects.
+        pass
+
     def list(self, **extra_args):
         """
         Return list of IDs of saved `Job` objects.
@@ -56,61 +104,46 @@ class Store(object):
         raise NotImplementedError(
             "Method `list` not implemented in this class.")
 
+    @abstractmethod
+    def load(self, id_):
+        """
+        Load a saved object given its ID, and return it.
+        """
+        pass
+
+    def _update_to_latest_schema(self):
+        """
+        Modify an object in-place to reflect changes in the schema.
+
+        Called immediately after a successful `load()`, just before returning
+        the retrieved object to the caller.
+        """
+        pass
+
+    @abstractmethod
     def remove(self, id_):
         """
         Delete a given object from persistent storage, given its ID.
         """
-        raise NotImplementedError(
-            "Abstract method 'Store.remove' called"
-            " -- should have been implemented in a derived class!")
+        pass
 
+    @abstractmethod
     def replace(self, id_, obj):
         """
         Replace the object already saved with the given ID with a copy
         of `obj`.
         """
-        raise NotImplementedError(
-            "Abstract method 'Store.replace' called"
-            " -- should have been implemented in a derived class!")
-
-    def load(self, id_):
-        """
-        Load a saved object given its ID, and return it.
-        """
-        raise NotImplementedError(
-            "Abstract method 'Store.load' called"
-            " -- should have been implemented in a derived class!")
+        pass
 
+    @abstractmethod
     def save(self, obj):
         """
         Save an object, and return an ID.
         """
-        raise NotImplementedError(
-            "Abstract method 'Store.save' called"
-            " -- should have been implemented in a derived class!")
+        pass
 
 
-class Persistable(object):
-    """
-    A mix-in class to mark that an object should be persisted by its ID.
-
-    Any instance of this class is saved as an 'external reference'
-    when a container holding a reference to it is saved.
-
-    """
-    def __init__(self, *args, **kwargs):
-        # ensure object will be saved next time Store.save() is invoked
-        self.changed = True
-
-    def __str__(self):
-        try:
-            return str(self.persistent_id)
-        except AttributeError:
-            return super(Persistable, self).__str__()
-
-
-
-## registration mechanism
+# registration mechanism
 
 _registered_store_ctors = {}
 
@@ -130,7 +163,7 @@ def register(scheme, constructor):
     :param str scheme: URL scheme to associate with the given constructor.
 
     :param callable constructor: A callable returning a `Store`:class:
-    instance. Typically, a class constructor.
+      instance. Typically, a class constructor.
     """
     global _registered_store_ctors
     assert callable(constructor), (
@@ -170,37 +203,49 @@ def make_store(uri, *args, **extra_args):
     """
     if not isinstance(uri, Url):
         uri = Url(uri)
-    # create and return store
+
+    # since SQLAlchemy allows URIs of the form `db+driver://...`
+    # (e.g., `postresql+psycopg://...`) we need to examine the URI
+    # scheme only up to the first `+`
+    scheme = uri.scheme.split('+')[0]
+
     try:
         # hard-code schemes that are supported by GC3Pie itself
         if uri.scheme == 'file':
             import gc3libs.persistence.filesystem
-            return gc3libs.persistence.filesystem.make_filesystemstore(uri, *args, **extra_args)
-        elif uri.scheme in [
-                # XXX: list all supported SQLAlchemy back-ends
+            return gc3libs.persistence.filesystem.make_filesystemstore(
+                uri, *args, **extra_args)
+        elif scheme in [
+                # DBs supported in SQLAlchemy core as of version 1.1,
+                # see: http://docs.sqlalchemy.org/en/latest/dialects/index.html
                 'firebird',
                 'mssql',
                 'mysql',
                 'oracle',
-                'postgres',
+                'postgresql',
                 'sqlite',
-                ]:
+                'sybase',
+        ]:
             import gc3libs.persistence.sql
-            return gc3libs.persistence.sql.make_sqlstore(uri, *args, **extra_args)
+            return gc3libs.persistence.sql.make_sqlstore(
+                uri, *args, **extra_args)
         else:
             try:
-                return _registered_store_ctors[uri.scheme](uri, *args, **extra_args)
+                return _registered_store_ctors[
+                    uri.scheme](uri, *args, **extra_args)
             except KeyError:
                 gc3libs.log.error(
-                    "Unknown URL scheme '%s' in `gc3libs.persistence.make_store`:"
+                    "Unknown URL scheme '%s' in"
+                    " `gc3libs.persistence.make_store`:"
                     " has never been registered.", uri.scheme)
                 raise
-    except Exception, ex:
-        gc3libs.log.error("Error constructing store for URL '%s': %s: %s",
-                          str(uri), ex.__class__.__name__, str(ex))
+    except Exception as err:
+        gc3libs.log.error(
+            "Error constructing store for URL '%s': %s: %s",
+            uri, err.__class__.__name__, err)
         raise
 
-## main: run tests
+# main: run tests
 
 if "__main__" == __name__:
     import doctest
