@@ -3,7 +3,7 @@
 """
 Test persistence backends.
 """
-# Copyright (C) 2011, 2012, GC3, University of Zurich. All rights reserved.
+# Copyright (C) 2011, 2012, 2019,  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -19,26 +19,23 @@ Test persistence backends.
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
+from __future__ import absolute_import, print_function, unicode_literals
+from builtins import str
+from builtins import range
+from builtins import object
 __docformat__ = 'reStructuredText'
-__version__ = '$Revision$'
 
 # stdlib imports
 import os
-import pickle
 import shutil
-import tempfile
+from tempfile import NamedTemporaryFile, mkdtemp
 
 # 3rd party imports
-from nose.plugins.skip import SkipTest
-from nose.tools import raises, assert_equal
-from nose import with_setup
+import pytest
 
-try:
-    import sqlalchemy
-    import sqlalchemy.sql as sql
-    sqlfunc = sqlalchemy.func
-except:
-    raise SkipTest("Cannot import `sqlalchemy`; skipping all SQL tests.")
+sqlalchemy = pytest.importorskip("sqlalchemy")
+sql = pytest.importorskip("sqlalchemy.sql")
+sqlfunc = sqlalchemy.func
 
 # GC3Pie imports
 import gc3libs
@@ -52,39 +49,65 @@ from gc3libs.persistence.serialization import DEFAULT_PROTOCOL
 from gc3libs.persistence.idfactory import IdFactory
 from gc3libs.persistence.filesystem import FilesystemStore
 from gc3libs.persistence.sql import SqlStore
-from gc3libs.url import Url
+from gc3libs.url import Url, UrlKeyDict
 
-def test_store_ctor_with_extra_arguments():
+
+@pytest.mark.parametrize("cls", (SqlStore, FilesystemStore))
+def test_store_ctor_with_extra_arguments(cls, tmpdir):
     """Test if `Store`:class: classess accept extra keyword arguments
     without complaining.
 
     This test will ensure that any `Store` class can be called with
     arguments which are valid only on other `Store` classes.
     """
-    tmpdir = tempfile.mkdtemp()
     args = {
-        'url': 'sqlite:///%s/test.sqlite' % os.path.abspath(tmpdir),
+        'url': 'sqlite:///%s/test.sqlite' %
+        os.path.abspath(str(tmpdir)),
         'table_name': 'store',
         'create': True,
-        'directory': tmpdir,
+        'directory': str(tmpdir),
         'idfactory': IdFactory(),
         'protocol': DEFAULT_PROTOCOL,
-        'extra_fields': {sqlalchemy.Column('extra',         sqlalchemy.TEXT())    : lambda x: "test"},
+        'extra_fields': {
+            sqlalchemy.Column(
+                'extra',
+                sqlalchemy.TEXT()): lambda x: "test"},
+    }
 
-        }
+    cls(**args)
 
-    def run_test(cls, args):
-        cls(**args)
 
-    for cls in (SqlStore, FilesystemStore):
-        yield run_test, cls, args
-        if os.path.exists(tmpdir):
-            shutil.rmtree(tmpdir)
+def test_eq_persisted_objects():
+    """Test that the comparison of two persisted objects are the same
+    whenever `persistent_id` attribute is the same"""
+    a = Persistable()
+    b = Persistable()
+    c = Persistable()
+    a.persistent_id = '1'
+    b.persistent_id = '1'
+    c.persistnet_id = '2'
+    assert a == b
+    assert a is not b
+    assert a != c
+    L = [a, b, c]
+    L.remove(a)
+    L.remove(a)
+    assert len(L) == 1
 
-## for testing basic functionality we do no need fully-fledged GC3Pie
-## objects; let's define some simple make-do's.
+def test_eq_non_persisted_objects():
+    """Test that the comparison of two not (yet) persisted objects falls
+    back to default Python comparison"""
+    a = Persistable()
+    b = Persistable()
+    assert a != b
+    assert a is not b
+
+
+# for testing basic functionality we do no need fully-fledged GC3Pie
+# objects; let's define some simple make-do's.
 
 class SimplePersistableObject(Persistable):
+
     def __init__(self, x):
         self.value = x
 
@@ -93,6 +116,7 @@ class SimplePersistableObject(Persistable):
 
 
 class SimplePersistableList(list, Persistable):
+
     """
     Add a `__dict__` to `list`, so that creating a `persistent_id`
     entry on an instance works.
@@ -106,6 +130,7 @@ class SimpleTask(Task):
 
 
 class NonPersistableClassWithSlots(object):
+
     """
     A class to test that persistence of classes with `__slots__`.
 
@@ -133,23 +158,26 @@ class PersistableClassWithSlots(NonPersistableClassWithSlots):
 
 
 class MyChunkedParameterSweep(gc3libs.workflow.ChunkedParameterSweep):
+
     def new_task(self, param, **extra_args):
         return Task(**extra_args)
 
 
 class MyStagedTaskCollection(gc3libs.workflow.StagedTaskCollection):
+
     def stage0(self):
         return Task()
 
 
-def _run_generic_tests(store):
-    _generic_persistence_test(store)
-    _generic_nested_persistence_test(store)
-    _generic_persist_classes_with_slots(store)
-    _save_different_objects_separated(store)
+# def _run_generic_tests(store):
+#     _generic_persistence_test(store)
+#     _generic_nested_persistence_test(store)
+#     _generic_persist_classes_with_slots(store)
+#     _save_different_objects_separated(store)
 
 
 class GenericStoreChecks(object):
+
     """
     A suite of tests for the generic `gc3libs.persistence.Store` interface.
     """
@@ -194,7 +222,6 @@ class GenericStoreChecks(object):
 
         assert len(self.store.list()) == num_objs
 
-    @raises(gc3libs.exceptions.LoadError)
     def test_remove_method(self):
         """
         Test remove method of a generic `Store` class
@@ -204,7 +231,8 @@ class GenericStoreChecks(object):
         id = self.store.save(obj)
         self.store.remove(id)
 
-        obj = self.store.load(id)
+        with pytest.raises(gc3libs.exceptions.LoadError):
+            obj = self.store.load(id)
 
     def test_replace_method(self):
         """Test the `replace` method of the `SqlStore` class"""
@@ -221,8 +249,8 @@ class GenericStoreChecks(object):
         obj2 = self.store.load(id_)
         assert obj2.x == "Updated"
 
+    @pytest.mark.skip(reason="FIXME: Test code needs to be checked!")
     def test_persist_classes_with_slots(self):
-        raise SkipTest("FIXME: Test code needs to be checked!")
 
         # FIXME: check this code!
         obj = PersistableClassWithSlots('GC3')
@@ -274,44 +302,57 @@ class GenericStoreChecks(object):
         # return objects for further testing
         return (container_id, objid)
 
-    def test_task_objects(self):
+    def test_persist_urlkeydict(self):
         """
-        Test that all `Task`-like objects are persistable
+        Test that we can persist GC3Pie's `UrlKeyDict` classes.
         """
-        def check_task(task):
-            fd, tmpfile = tempfile.mkstemp()
-            store = make_store("sqlite://%s" % tmpfile)
-            try:
-                id = store.save(task)
-                store.load(id)
-            finally:
-                os.remove(tmpfile)
+        a = Task(attr=UrlKeyDict({'/tmp/1': 1, '/tmp/2': 2}))
+        id_ = self.store.save(a)
+        b = self.store.load(id_)
+        assert b.attr == a.attr
 
-        for obj in [
-            Task(),
-            gc3libs.workflow.TaskCollection(tasks=[Task(), Task()]),
-            gc3libs.workflow.SequentialTaskCollection([Task(), Task()]),
-            MyStagedTaskCollection(),
-            gc3libs.workflow.ParallelTaskCollection(tasks=[Task(), Task()]),
-            MyChunkedParameterSweep(1, 20, 1, 5),
-            gc3libs.workflow.RetryableTask(Task()),
-            ]:
-            check_task.description = "Test that Task-like `%s` class is stored correctly" %  obj.__class__.__name__
-            yield check_task, obj
+
+    def test_persist_urlvaluedict(self):
+        """
+        Test that we can persist GC3Pie's `UrlValueDict` classes.
+        """
+        a = Task(attr=UrlKeyDict({'foo': '/tmp/1', 'bar': '/tmp/2'}))
+        id_ = self.store.save(a)
+        b = self.store.load(id_)
+        assert b.attr == a.attr
+
+
+@pytest.mark.parametrize("task", (Task(),
+        gc3libs.workflow.TaskCollection(tasks=[Task(), Task()]),
+        gc3libs.workflow.SequentialTaskCollection([Task(), Task()]),
+        MyStagedTaskCollection(),
+        gc3libs.workflow.ParallelTaskCollection(tasks=[Task(), Task()]),
+        MyChunkedParameterSweep(1, 20, 1, 5),
+        gc3libs.workflow.RetryableTask(Task())))
+def test_task_objects_buggy(task):
+    """
+    Test that all `Task`-like objects are persistable
+    """
+    with NamedTemporaryFile(prefix='gc3libs.', suffix='.tmp') as tmp:
+        store = make_store("sqlite://%s" % tmp.name)
+        id = store.save(task)
+        store.load(id)
 
 
 class TestFilesystemStore(GenericStoreChecks):
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
         from gc3libs.persistence.filesystem import FilesystemStore
-        self.tmpdir = tempfile.mkdtemp()
+        self.tmpdir = mkdtemp(prefix='gc3libs.', suffix='.tmp.d')
         self.store = FilesystemStore(self.tmpdir)
 
-    def tearDown(self):
+        yield
+
         shutil.rmtree(self.tmpdir)
 
     # XXX: there's nothing which is `FilesystemStore`-specific here!
-    def test_filesystemstorage_pickler_class(self):
+    def check_file_accesssystemstorage_pickler_class(self):
         """
         Check that `Persistable` instances are saved saparately.
 
@@ -344,7 +385,8 @@ class TestFilesystemStore(GenericStoreChecks):
         also test that files have been created.
         """
         # std checks
-        container_id, obj_id = super(TestFilesystemStore, self).test_disaggregate_persistable_objects()
+        container_id, obj_id = super(
+            TestFilesystemStore, self).test_disaggregate_persistable_objects()
         # check that files exist
         container_file = os.path.join(self.store._directory, str(container_id))
         assert os.path.exists(container_file)
@@ -353,6 +395,7 @@ class TestFilesystemStore(GenericStoreChecks):
 
 
 class SqlStoreChecks(GenericStoreChecks):
+
     """
     Extend `GenericStoreChecks` with additional SQL-related tests.
 
@@ -377,8 +420,8 @@ class SqlStoreChecks(GenericStoreChecks):
         id_ = self.store.save(app)
 
         q = sql.select([
-            self.store.t_store.c.state
-            ]).where(self.store.t_store.c.id == id_)
+            self.store._tables.c.state
+        ]).where(self.store._tables.c.id == id_)
         result = self.conn.execute(q)
         row = result.fetchone()
 
@@ -397,8 +440,8 @@ class SqlStoreChecks(GenericStoreChecks):
         id_ = self.store.save(app)
 
         q = sql.select([
-            self.store.t_store.c.state
-            ]).where(self.store.t_store.c.id == id_)
+            self.store._tables.c.state
+        ]).where(self.store._tables.c.id == id_)
         result = self.conn.execute(q)
         row = result.fetchone()
         assert row[0] == app.execution.state
@@ -406,11 +449,15 @@ class SqlStoreChecks(GenericStoreChecks):
     def test_sql_injection(self):
         """Test if the `SqlStore` class is vulnerable to SQL injection."""
 
-        storetemp = make_store(self.db_url,
+        storetemp = make_store(
+            self.db_url,
             table_name='sql_injection_test',
             extra_fields={
-                sqlalchemy.Column('extra', sqlalchemy.VARCHAR(length=128)): GET.extra,
-                })
+                sqlalchemy.Column(
+                    'extra',
+                    sqlalchemy.VARCHAR(
+                        length=128)): GET.extra,
+            })
 
         obj = SimpleTask()
         # obligatory XKCD citation ;-)
@@ -428,18 +475,22 @@ class SqlStoreChecks(GenericStoreChecks):
         """
         # extend the db
         self.conn.execute('alter table `%s` add column extra varchar(256)'
-                       % self.store.table_name)
+                          % self.store.table_name)
 
         # re-build store, as the table list is read upon `__init__`
-        self.store = self._make_store(extra_fields={
-            sqlalchemy.Column('extra', sqlalchemy.VARCHAR(length=128)): GET.foo.value,
+        self.store = self._make_store(
+            extra_fields={
+                sqlalchemy.Column(
+                    'extra',
+                    sqlalchemy.VARCHAR(
+                        length=128)): GET.foo.value,
             })
 
         # if this query does not error out, the column is defined
-        q = sql.select([sqlfunc.count(self.store.t_store.c.extra)]).distinct()
+        q = sql.select([sqlfunc.count(self.store._tables.c.extra)]).distinct()
         results = self.conn.execute(q)
         rows = results.fetchall()
-        assert_equal(len(rows), 1)
+        assert len(rows) == 1
 
         # create and save an object
         obj = SimplePersistableObject('an object')
@@ -447,28 +498,35 @@ class SqlStoreChecks(GenericStoreChecks):
         id_ = self.store.save(obj)
 
         # check that the value has been saved
-        q = sql.select([self.store.t_store.c.extra]).where(self.store.t_store.c.id == id_)
+        q = sql.select([self.store._tables.c.extra]).where(
+            self.store._tables.c.id == id_)
 
         # Oops, apparently the store.save call will close our
         # connection too.
         self.conn = self.store._engine.connect()
         results = self.conn.execute(q)
         rows = results.fetchall()
-        assert_equal(len(rows), 1)
-        assert_equal(rows[0][0], obj.foo.value)
+        assert len(rows) == 1
+        assert rows[0][0] == obj.foo.value
 
-    @raises(AssertionError)
+    @pytest.mark.skip(reason="FIXME: Check if test is still valid")
     def test_sql_error_if_no_extra_fields(self):
         """
         Test if `SqlStore` reads and writes extra columns.
         """
-        # re-build store with a non-existent extra column; should raise `AssertionError`
-        raise SkipTest("Feature not supported anymore")
-        self._make_store(extra_fields={
-            sqlalchemy.Column('extra', sqlalchemy.VARCHAR(length=128)): (lambda arg: arg.foo.value)})
+        # re-build store with a non-existent extra column
+        with pytest.raises(AssertionError):
+            self._make_store(
+                extra_fields={
+                    sqlalchemy.Column(
+                        'extra',
+                        sqlalchemy.VARCHAR(
+                            length=128)): (
+                                lambda arg: arg.foo.value)})
 
 
 class ExtraSqlChecks(object):
+
     """
     `SqlStore` that depend on the setup of a non-default table.
     """
@@ -484,80 +542,71 @@ class ExtraSqlChecks(object):
         """
 
         # extend the db
-        self._make_store(self.db_url,
-                         extra_fields={sqlalchemy.Column('extra', sqlalchemy.VARCHAR(length=128)): (lambda arg: arg.foo.value)})
+        self._make_store(
+            self.db_url,
+            extra_fields={
+                sqlalchemy.Column(
+                    'extra',
+                    sqlalchemy.VARCHAR(
+                        length=128)): (
+                    lambda arg: arg.foo.value)})
 
         # if this query does not error out, the column is defined
-        q = sql.select([sqlfunc.count(self.store.t_store.c.extra)]).distinct()
+        q = sql.select([sqlfunc.count(self.store._tables.c.extra)]).distinct()
         results = self.conn.execute(q)
-        # self.c.execute("select distinct count(extra) from %s" % self.store.table_name)
         rows = results.fetchall()
-        assert_equal(len(rows), 1)
+        assert len(rows) == 1
 
         # create and save an object
         obj = SimplePersistableObject('an object')
         obj.foo = SimplePersistableObject('an attribute')
-        id_ = db.save(obj)
+        id_ = self.store.save(obj)
 
         # check that the value has been saved
-        q = sql.select([self.store.t_store.c.extra]).where(self.store.t_store.c.id == id_)
+        q = sql.select([self.store._tables.c.extra]).where(
+            self.store._tables.c.id == id_)
         # self.c.execute("select extra from %s where id=%d"
         #                % (self.store.table_name, id_))
         results = self.conn.execute(q)
         rows = results.fetchall()
-        assert_equal(len(rows), 1)
-        assert_equal(rows[0][0], obj.foo.value)
+        assert len(rows) == 1
+        assert rows[0][0] == obj.foo.value
 
 
 class TestSqliteStore(SqlStoreChecks):
+
     """Test SQLite backend."""
 
     @classmethod
     def setup_class(cls):
         # skip SQLite tests if no SQLite module present (Py 2.4)
-        try:
-            import sqlite3
-        except ImportError:
-            # SQLAlchemy uses `pysqlite2` on Py 2.4
-            try:
-                import pysqlite2
-            except ImportError:
-                raise SkipTest("No SQLite module installed.")
-
-    @classmethod
-    def teardown_class(cls):
-        pass
+        sqlite3 = pytest.importorskip("sqlite3")
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
-        fd, self.tmpfile = tempfile.mkstemp()
-        self.db_url = Url('sqlite://%s' % self.tmpfile)
-        self.store = self._make_store()
+        with NamedTemporaryFile(prefix='gc3libs.', suffix='.tmp') as tmp:
+            self.tmpfile = tmp.name
+            self.db_url = Url('sqlite://%s' % self.tmpfile)
+            self.store = self._make_store()
 
-        # create a connection to the database
-        self.conn = self.store._engine.connect()
+            # create a connection to the database
+            self.conn = self.store._engine.connect()
 
-    def tearDown(self):
-        self.conn.close()
-        os.remove(self.tmpfile)
+            yield
+            self.conn.close()
 
     def _make_store(self, **kwargs):
         return make_store(self.db_url, **kwargs)
 
 
 class TestSqliteStoreWithAlternateTable(TestSqliteStore):
+
     """Test SQLite backend with a different table name."""
 
     @classmethod
     def setup_class(cls):
         # skip SQLite tests if no SQLite module present (Py 2.4)
-        try:
-            import sqlite3
-        except ImportError:
-            # SQLAlchemy uses `pysqlite2` on Py 2.4
-            try:
-                import pysqlite2
-            except ImportError:
-                raise SkipTest("No SQLite module installed.")
+        sqlite3 = pytest.importorskip("sqlite3")
 
     def _make_store(self, **kwargs):
         return make_store(self.db_url, table_name='another_store', **kwargs)
@@ -566,7 +615,7 @@ class TestSqliteStoreWithAlternateTable(TestSqliteStore):
         obj = SimplePersistableObject('an object')
         self.store.save(obj)
 
-        results = self.conn.execute(sql.select([self.store.t_store.c.id]))
+        results = self.conn.execute(sql.select([self.store._tables.c.id]))
         rows = results.fetchall()
         assert len(rows) > 0
 
@@ -576,30 +625,30 @@ class TestMysqlStore(SqlStoreChecks):
     @classmethod
     def setup_class(cls):
         # we skip MySQL tests if no MySQLdb module is present
-        try:
-            import MySQLdb
-        except:
-            raise SkipTest("MySQLdb module not installed.")
+        MySQLdb = pytest.importorskip("MySQLdb")
 
     @classmethod
     def teardown_class(cls):
         pass
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
-        fd, tmpfile = tempfile.mkstemp()
-        os.remove(tmpfile)
-        self.table_name = tmpfile.split('/')[-1]
+        # generate random table name
+        from string import ascii_letters as letters
+        import random
+        self.table_name = 'test_' + (''.join(random.choice(letters) for _ in range(10)))
 
         try:
             self.db_url = Url('mysql://gc3user:gc3pwd@localhost/gc3')
             self.store = make_store(self.db_url, table_name=self.table_name)
         except sqlalchemy.exc.OperationalError:
-            raise SkipTest("Cannot connect to MySQL database.")
+            pytest.mark.skip("Cannot connect to MySQL database.")
 
         # create a connection to the database
         self.conn = self.store._engine.connect()
 
-    def tearDown(self):
+        yield
+
         self.conn.execute('drop table `%s`' % self.table_name)
         self.conn.close()
         # self.c.close()
@@ -610,5 +659,4 @@ class TestMysqlStore(SqlStoreChecks):
 
 
 if __name__ == "__main__":
-    import nose
-    nose.runmodule()
+    pytest.main(["-v", __file__])
