@@ -5,10 +5,11 @@ Manipulation of quantities with units attached with automated
 conversion among compatible units.
 
 For details and the discussion leading up to this,
-see: `<http://code.google.com/p/gc3pie/issues/detail?id=47>`
+see: `<https://github.com/uzh/gc3pie/issues/47>`
 
 """
-# Copyright (C) 2011, 2012, 2013, GC3, University of Zurich. All rights reserved.
+
+# Copyright (C) 2011 - 2014, 2019,  University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -21,24 +22,37 @@ see: `<http://code.google.com/p/gc3pie/issues/detail?id=47>`
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-#
-__docformat__ = 'reStructuredText'
-__version__ = '$Revision$'
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import absolute_import, print_function, unicode_literals
+from __future__ import division
+from builtins import object
+from past.utils import old_div
+
+# future's own `with_metaclass` does not play well with the `Quantity`
+# implementation here, because `Quantity` defines its own `__init__`
+# and `__call__` , which `future.utils.with_metaclass` overrides with
+# the std ones coming from Python's builtin `type`
+from six import add_metaclass
 
 # stdlib imports
 import datetime
 import operator
 import re
-import types
 
-# GC3Libs imports
-from gc3libs.utils import defproperty
+try:
+    # Python 2
+    from types import StringTypes as string_types
+except ImportError:
+    # Python 3
+    string_types = (str,)
 
 
-## utility functions
+# module metadata
+__docformat__ = 'reStructuredText'
+
+
+# utility functions
 
 _QTY_RE = re.compile(
     r'(?P<amount>[+-]?([0-9]+(\.[0-9]+)?|\.[0-9]+)(E[+-]?[0-9]+)?)'
@@ -46,6 +60,7 @@ _QTY_RE = re.compile(
     r'(?P<unit>[a-z]+)?',
     re.I | re.X)
 
+
 def _split_amount_and_unit(val, default_unit=None, allow=None):
     """
     Split `val` into amount and measurement unit.
@@ -54,12 +69,12 @@ def _split_amount_and_unit(val, default_unit=None, allow=None):
     unit specification. The number and the unit may be separated by 0
     or more spaces.
 
-      >>> _split_amount_and_unit('7 s')
-      (7.0, 's')
-      >>> _split_amount_and_unit('7s')
-      (7.0, 's')
-      >>> _split_amount_and_unit('-7.0s')
-      (-7.0, 's')
+      >>> _split_amount_and_unit('7 s') == (7.0, 's')
+      True
+      >>> _split_amount_and_unit('7s') ==  (7.0, 's')
+      True
+      >>> _split_amount_and_unit('-7.0s') == (-7.0, 's')
+      True
 
     If `val` does not conform to this syntax, a `ValueError` will
     be raised::
@@ -72,15 +87,15 @@ def _split_amount_and_unit(val, default_unit=None, allow=None):
     If string `val` only specifies an amount (e.g., ``42``) with no
     unit, then the optional argument `default_unit` provides one::
 
-      >>> _split_amount_and_unit('7', 'min')
-      (7.0, 'min')
+      >>> _split_amount_and_unit('7', 'min') == (7.0, 'min')
+      True
 
     Note the amount is a floating point number, whereas the unit is
     always a string.  By default, no validity checks are performed
     on the 'unit' part, which can be any word::
 
-      >>> _split_amount_and_unit('7sins')
-      (7.0, 'sins')
+      >>> _split_amount_and_unit('7sins') == (7.0, 'sins')
+      True
 
     The optional argument `allow` restricts the resulting quantity to
     be a one of the specified units; if it's not, a `ValueError`
@@ -101,8 +116,10 @@ def _split_amount_and_unit(val, default_unit=None, allow=None):
         unit = default_unit
     if allow is not None:
         if unit not in allow:
-            raise ValueError("Unit '%s' is not allowed here: only %s are."
-                             % (unit, str.join(",", [("'%s'" % a) for a in sorted(allow)])))
+            raise ValueError(
+                "Unit '%s' is not allowed here: only %s are." %
+                (unit, ",".join([
+                    ("'%s'" % a) for a in sorted(allow)])))
     return (amount, unit)
 
 
@@ -123,12 +140,14 @@ def _make_comparison_function(op, domain):
                 "Cannot compare '%s' with '%s':"
                 " Can only compare homogeneous quantities!"
                 % (self.__class__.__name__, other.__class__.__name__))
-            return op(self.amount(unit=self.base, conv=domain), other.amount(unit=self.base, conv=domain))
+            return op(self.amount(unit=self.base, conv=domain),
+                      other.amount(unit=self.base, conv=domain))
         return cmp_fn
     return decorate
 
 
 class _Quantity(object):
+
     """
     Represent a dimensioned value.
 
@@ -166,7 +185,7 @@ class _Quantity(object):
 
       - http://martinfowler.com/eaaDev/quantity.html
       - http://mail.python.org/pipermail/python-ideas/2010-March/006894.html
-      - http://mgrzyb.blogspot.ch/2011/08/quantity-pattern-lessons-learned-from.html
+      - http://goo.gl/Drta6i
 
     """
 
@@ -175,8 +194,7 @@ class _Quantity(object):
         '_base',
         '_name',
         '_unit',
-        '_UNITS',
-        )
+    )
 
     def amount(self, unit=None, conv=(lambda value: value)):
         """
@@ -194,10 +212,10 @@ class _Quantity(object):
         """
         if unit is None:
             unit = self.unit
-        return (conv(self._amount) / conv(unit._amount))
+        return (old_div(conv(self._amount), conv(unit._amount)))
 
-    @defproperty
-    def base():
+    @property
+    def base(self):
         """
         Return the base unit for all quantities in this class.
         The amount of any quantity is internally stored as a multiple
@@ -205,24 +223,20 @@ class _Quantity(object):
 
         This is a read-only class-level attribute.
         """
-        def fget(self):
-            return self._base
-        return locals()
+        return self._base
 
-    @defproperty
-    def unit():
+    @property
+    def unit(self):
         """
         The unit the amount of this quantity is expressed in.
 
         This is a read-only attribute: once set in the constructor, it
         cannot be changed.
         """
-        def fget(self):
-            return self._unit
-        return locals()
+        return self._unit
 
-    @defproperty
-    def name():
+    @property
+    def name(self):
         """
         The name of this quantity, or `None`.
 
@@ -232,13 +246,11 @@ class _Quantity(object):
         This is a read-only attribute: once set in the constructor, it
         cannot be changed.
         """
-        def fget(self):
-            return self._name
-        return locals()
+        return self._name
 
-    ## instance construction
+    # instance construction
 
-    _UNITS = { }
+    _UNITS = {}
     """
     A registry of valid units (by name).
 
@@ -254,12 +266,11 @@ class _Quantity(object):
         """
         cls._UNITS[unit.name] = unit
 
-
     # since we are subclassing a builtin type,
     # we need to provide `__new__`, not `__init__`
     def __new__(cls, val, unit=None, name=None):
         # dispatch to actual constructor depending on the type of `val`
-        if isinstance(val, types.StringTypes):
+        if isinstance(val, string_types):
             new = cls._new_from_string(val)
         elif isinstance(val, _Quantity):
             new = cls._new_from_amount_and_unit(val.amount(), val.unit)
@@ -270,11 +281,12 @@ class _Quantity(object):
             new._unit = new
         else:
             # default: quantity is int(val)*unit
-            assert unit is not None, ("Cannot construct a quantity from amount alone.")
+            assert unit is not None, (
+                "Cannot construct a quantity from amount alone.")
             assert unit.name in cls._UNITS, (
                 "Unit '%s' not allowed in '%s' quantity: only %s are."
                 % (unit.name, cls.__name__,
-                   str.join(',', [("'%s'" % u) for u in cls._UNITS])))
+                   ','.join([("'%s'" % u) for u in cls._UNITS])))
             new = cls._new_from_amount_and_unit(val, unit)
         new._name = name
         if name is not None:
@@ -284,7 +296,7 @@ class _Quantity(object):
     @classmethod
     def _new_from_amount_and_unit(cls, amount, unit):
         new = super(_Quantity, cls).__new__(cls)
-        new._amount = amount*unit.amount(cls._base)
+        new._amount = amount * unit.amount(cls._base)
         new._unit = unit
         new._name = None
         return new
@@ -301,8 +313,7 @@ class _Quantity(object):
         # be the very class itself that we are going to serialize)
         return (("%d %s" % (self.amount(), self.unit.name)), self.name)
 
-
-    ## string representation
+    # string representation
     def to_str(self, fmt, unit=None, conv=(lambda value: value)):
         """
         Return a string representation of the quantity.
@@ -331,7 +342,7 @@ class _Quantity(object):
             unit = self.unit
         try:
             return (fmt % (self.amount(unit, conv=conv), unit.name))
-        except TypeError: # not all arguments converted
+        except TypeError:  # not all arguments converted
             return (fmt % self.amount(unit, conv=conv)) + unit.name
 
     def __str__(self):
@@ -341,15 +352,21 @@ class _Quantity(object):
         return self.to_str("%g%s")
 
     def __repr__(self):
-        """Return a string representation that can be read back with `eval()`."""
+        """
+        Return a string representation that can be read back with
+        `eval()`.
+        """
         if hasattr(self.unit, 'name'):
             return ('%s(%g, unit=%s)'
                     % (self.__class__.__name__, self.amount(), self.unit.name))
         else:
-            return ('%s(%g, unit=<%s object at 0x%x>)'
-                    % (self.__class__.__name__, self.amount(), self.unit.__class__.__name__, id(self.unit)))
+            return (
+                '%s(%g, unit=<%s object at 0x%x>)' %
+                (self.__class__.__name__, self.amount(),
+                 self.unit.__class__.__name__, id(self.unit)))
 
-    ## arithmetic: allow multiplication by a scalar, and division by a quantity (of the same kind)
+    # arithmetic: allow multiplication by a scalar, and division by a quantity
+    # (of the same kind)
     @staticmethod
     def _smallest_unit(self, other):
         """
@@ -378,22 +395,24 @@ class _Quantity(object):
 
     def __add__(self, other):
         assert isinstance(other, self.__class__), \
-               ("Cannot add '%s' to '%s':"
+            ("Cannot add '%s' to '%s':"
                 " can sum only homogeneous quantities."
                 % (self.__class__.__name__, other.__class__.__name__))
         unit = self._smallest_unit(self, other)
         return self._new_from_amount_and_unit(
-            (self.amount(self.base) + other.amount(self.base)) / unit.amount(self.base),
+            old_div((self.amount(self.base) + other.amount(self.base)),
+            unit.amount(self.base)),
             unit=unit)
 
     def __sub__(self, other):
         assert isinstance(other, self.__class__), \
-               ("Cannot subtract '%s' from '%s':"
+            ("Cannot subtract '%s' from '%s':"
                 " can only operate on homogeneous quantities."
                 % (self.__class__.__name__, other.__class__.__name__))
         unit = self._smallest_unit(self, other)
         return self._new_from_amount_and_unit(
-            (self.amount(self.base) - other.amount(self.base)) / unit.amount(self.base),
+            old_div((self.amount(self.base) - other.amount(self.base)),
+            unit.amount(self.base)),
             unit=unit)
 
     def __mul__(self, coeff):
@@ -403,9 +422,9 @@ class _Quantity(object):
             except (TypeError, ValueError):
                 raise TypeError("Cannot multiply '%s' and '%s':"
                                 " can only multiply '%s' by a pure number."
-                                ""% (coeff.__class__.__name__,
-                                     self.__class__.__name__,
-                                     self.__class__.__name__))
+                                "" % (coeff.__class__.__name__,
+                                      self.__class__.__name__,
+                                      self.__class__.__name__))
         return self._new_from_amount_and_unit(coeff * self.amount(), self.unit)
     __rmul__ = __mul__
 
@@ -415,32 +434,40 @@ class _Quantity(object):
         or divide a quantity by the specified amount.
         """
         try:
-            # the quotient of two (homogeneous) quantities is a ratio (pure number)
-            return (self.amount(self.base, conv=float) / other.amount(self.base, conv=float))
+            # the quotient of two (homogeneous) quantities is a ratio (pure
+            # number)
+            return (old_div(self.amount(self.base, conv=float),
+                    other.amount(self.base, conv=float)))
         except AttributeError:
             # we could really return `self * (1.0/other)`, but we want
             # to set the unit to a possibly smaller one (see
             # `_get_best_unit` above) to have a better "human" representation
             try:
-                amount = self.amount(self.base, conv=float) / float(other)
+                amount = old_div(self.amount(self.base, conv=float), float(other))
             except TypeError:
                 raise TypeError(
-                    "Cannot multiply '%s' and '%s': can only take"
+                    "Cannot divide '%s' by '%s': can only take"
                     " the ratio of '%s' and a pure number or an"
-                    " homogeneous quantity." % (coeff.__class__.__name__,
-                                                self.__class__.__name__,
+                    " homogeneous quantity." % (self.__class__.__name__,
+                                                other.__class__.__name__,
                                                 self.__class__.__name__))
             unit = self._largest_nonfractional_unit(amount)
-            return self._new_from_amount_and_unit(amount / unit.amount(self.base), unit)
+            return self._new_from_amount_and_unit(
+                old_div(amount, unit.amount(self.base)), unit)
 
+    # be compatible with `from __future__ import division`
+    __truediv__ = __div__
+
+    # FIXME: why doesn't this do the same as `__div__` (+ rounding down?)
     def __floordiv__(self, other):
         """Return the ratio of two quantities (as a whole number)."""
         assert isinstance(other, self.__class__), \
-               ("Cannot divide '%s' by '%s':"
-                " can only take the ratio of homogeneous quantities."
+            ("Cannot divide '%s' by '%s':"
+             " can only take the ratio of homogeneous quantities."
                 % (self.__class__.__name__, other.__class__.__name__))
         # the quotient of two (homogeneous) quantities is a ratio (pure number)
-        return (self.amount(self.base, conv=int) / other.amount(self.base, conv=int))
+        return (old_div(self.amount(self.base, conv=int),
+                other.amount(self.base, conv=int)))
 
     def __radd__(self, other):
         """
@@ -458,7 +485,8 @@ class _Quantity(object):
                 "Unsupported operands for +: %s (type '%s') and %s (type '%s')"
                 % (other, type(other), self, type(self)))
 
-    ## rich comparison operators, to ensure only homogeneous quantities are compared
+    # rich comparison operators, to ensure only homogeneous quantities are
+    # compared
     @_make_comparison_function(operator.gt, int)
     def __gt__(self, other):
         pass
@@ -485,6 +513,7 @@ class _Quantity(object):
 
 
 class Quantity(object):
+
     """
     Metaclass for creating quantity classes.
 
@@ -494,8 +523,9 @@ class Quantity(object):
     The name of the base unit is given as argument to the metaclass
     instance::
 
-      >>> class Memory1(object):
-      ...   __metaclass__ = Quantity('B')
+      >>> @add_metaclass(Quantity('B'))
+      ... class Memory1(object):
+      ...   pass
       ...
       >>> B = Memory1('1 B')
       >>> print (2*B)
@@ -505,8 +535,9 @@ class Quantity(object):
     key gives the unit name, and its value gives the ratio of the new
     unit to the base unit.  For example::
 
-      >>> class Memory2(object):
-      ...   __metaclass__ = Quantity('B', kB=1000, MB=1000*1000)
+      >>> @add_metaclass(Quantity('B', kB=1000, MB=1000*1000))
+      ... class Memory2(object):
+      ...   pass
       ...
       >>> a_thousand_kB = Memory2('1000kB')
       >>> MB = Memory2('1   MB')
@@ -520,6 +551,7 @@ class Quantity(object):
       True
 
     """
+
     def __init__(self, base_unit_name, **other_units):
         self.base_unit_name = base_unit_name
         self.other_units = other_units
@@ -535,7 +567,7 @@ class Quantity(object):
         newcls.register(base)
         setattr(newcls, self.base_unit_name, base)
         # create additional units and add them as class attributes
-        for name, amount in self.other_units.iteritems():
+        for name, amount in self.other_units.items():
             unit = newcls(amount, unit=newcls._base, name=name)
             # make new units default to self when printing the amount
             unit._unit = unit
@@ -544,7 +576,25 @@ class Quantity(object):
         return newcls
 
 
+@add_metaclass(Quantity(
+        # base unit is "bytes"; use the symbol 'B', although this is not the SI
+        # usage.
+        'B',
+        # 10-base units
+        kB=1000,
+        MB=1000 * 1000,
+        GB=1000 * 1000 * 1000,
+        TB=1000 * 1000 * 1000 * 1000,
+        PB=1000 * 1000 * 1000 * 1000 * 1000,
+        # binary base units
+        KiB=1024,                # KiBiByte
+        MiB=1024 * 1024,           # MiBiByte
+        GiB=1024 * 1024 * 1024,      # GiBiByte
+        TiB=1024 * 1024 * 1024 * 1024,  # TiBiByte
+        PiB=1024 * 1024 * 1024 * 1024 * 1024,  # PiBiByte
+    ))
 class Memory(object):
+
     """
     Represent an amount of RAM.
 
@@ -592,8 +642,10 @@ class Memory(object):
         >>> two_bytes == 2*byte
         True
         >>> half_gigabyte = a_gigabyte / 2
-        >>> half_gigabyte
-        Memory(476.837, unit=MiB)
+        >>> a_gigabyte == half_gigabyte * 2
+        True
+        >>> a_megabyte == a_gigabyte / 1000
+        True
 
     The ratio of two memory quantities is correctly computed as a pure
     (floating-point) number::
@@ -639,27 +691,27 @@ class Memory(object):
     By default, the unit used originally for defining the quantity is
     used::
 
-        >>> a_megabyte.to_str('%d [%s]')
-        '1 [MB]'
+        >>> '1 [MB]' == a_megabyte.to_str('%d [%s]')
+        True
 
     This can be overridden by specifying an optional second argument
     `unit`::
 
-        >>> a_megabyte.to_str('%d [%s]', unit=Memory.kB)
-        '1000 [kB]'
+        >>> '1000 [kB]' == a_megabyte.to_str('%d [%s]', unit=Memory.kB)
+        True
 
     A third optional argument `conv` can set the numerical type to be
     used for conversion computations::
 
-        >>> a_megabyte.to_str('%g%s', unit=Memory.GB, conv=float)
-        '0.001GB'
+        >>> '0.001GB' == a_megabyte.to_str('%g%s', unit=Memory.GB, conv=float)
+        True
 
     The default numerical type is `int`, which in particular implies
     that you get a null amount if the requested unit is larger than
     the quantity::
 
-        >>> a_megabyte.to_str('%g%s', unit=Memory.GB, conv=int)
-        '0GB'
+        >>> '0GB' == a_megabyte.to_str('%g%s', unit=Memory.GB, conv=int)
+        True
 
     Conversion to string uses the unit originally used for defining
     the quantity and the ``%g%s`` format::
@@ -668,25 +720,53 @@ class Memory(object):
         '1MB'
 
     """
-    __metaclass__ = Quantity(
-        # base unit is "bytes"; use the symbol 'B', although this is not the SI usage.
-        'B',
-        # 10-base units
-        kB=1000,
-        MB=1000*1000,
-        GB=1000*1000*1000,
-        TB=1000*1000*1000*1000,
-        PB=1000*1000*1000*1000*1000,
-        # binary base units
-        KiB=1024,                # KiBiByte
-        MiB=1024*1024,           # MiBiByte
-        GiB=1024*1024*1024,      # GiBiByte
-        TiB=1024*1024*1024*1024, # TiBiByte
-        PiB=1024*1024*1024*1024*1024, # PiBiByte
-        )
 
 
+@add_metaclass(Quantity(
+        # base unit is nanoseconds; use the SI symbol 'ns'
+        'ns',
+        # alternate spellings
+        nanosec=1,
+        nanosecond=1,
+        nanoseconds=1,
+        # microsecond(s)
+        us=1000,  # approx SI symbol
+        microsec=1000,
+        microseconds=1000,
+        # millisecond(s)
+        ms=10 ** 6,
+        millisec=10 ** 6,
+        milliseconds=10 ** 6,
+        # seconds(s)
+        s=10 ** 9,
+        sec=10 ** 9,
+        secs=10 ** 9,
+        second=10 ** 9,
+        seconds=10 ** 9,
+        # minute(s)
+        m=60 * 10 ** 9,
+        min=60 * 10 ** 9,
+        mins=60 * 10 ** 9,
+        minute=60 * 10 ** 9,
+        minutes=60 * 10 ** 9,
+        # hour(s)
+        h=60 * 60 * 10 ** 9,
+        hr=60 * 60 * 10 ** 9,
+        hrs=60 * 60 * 10 ** 9,
+        hour=60 * 60 * 10 ** 9,
+        hours=60 * 60 * 10 ** 9,
+        # day(s)
+        d=24 * 60 * 60 * 10 ** 9,
+        day=24 * 60 * 60 * 10 ** 9,
+        days=24 * 60 * 60 * 10 ** 9,
+        # week(s)
+        w=7 * 24 * 60 * 60 * 10 ** 9,
+        wk=7 * 24 * 60 * 60 * 10 ** 9,
+        week=7 * 24 * 60 * 60 * 10 ** 9,
+        weeks=7 * 24 * 60 * 60 * 10 ** 9,
+    ))
 class Duration(object):
+
     """
     Represent the duration of a time lapse.
 
@@ -740,10 +820,10 @@ class Duration(object):
       However, the formats HH:MM and MM:SS are rejected as ambiguous::
 
         >>> # is this hours:minutes or minutes:seconds ?
-        >>> l0 = Duration('01:02')
+        >>> l0 = Duration('01:02')  # doctest:+ELLIPSIS
         Traceback (most recent call last):
           ...
-        ValueError: Duration '01:02' is ambiguous: use '1m 2s' for 1 minutes and 2 seconds, or '1h 2m' for 1 hours and 2 minutes.
+        ValueError: Duration '01:02' is ambiguous: use '1m 2s' ...
 
     * Finally, you can specify a duration like any other quantity,
       as an integral amount of a given time unit::
@@ -800,6 +880,8 @@ class Duration(object):
       >>> two_hours = an_hour + an_hour
       >>> two_hours == 2*an_hour
       True
+      >>> an_hour == two_hours / 2
+      True
 
       >>> one_hour = two_hours - an_hour
       >>> one_hour.amount(seconds)
@@ -843,27 +925,27 @@ class Duration(object):
     By default, the unit used originally for defining the quantity is
     used::
 
-        >>> an_hour.to_str('%d [%s]')
-        '1 [hour]'
+        >>> '1 [hour]' == an_hour.to_str('%d [%s]')
+        True
 
     This can be overridden by specifying an optional second argument
     `unit`::
 
-        >>> an_hour.to_str('%d [%s]', unit=Duration.m)
-        '60 [m]'
+        >>> '60 [m]' == an_hour.to_str('%d [%s]', unit=Duration.m)
+        True
 
     A third optional argument `conv` can set the numerical type to be
     used for conversion computations::
 
-        >>> an_hour.to_str('%.1f [%s]', unit=Duration.m, conv=float)
-        '60.0 [m]'
+        >>> '60.0 [m]' == an_hour.to_str('%.1f [%s]', unit=Duration.m, conv=float)
+        True
 
     The default numerical type is `int`, which in particular implies
     that you get a null amount if the requested unit is larger than
     the quantity::
 
-        >>> an_hour.to_str('%d [%s]', unit=Duration.days)
-        '0 [days]'
+        >>> '0 [days]' == an_hour.to_str('%d [%s]', unit=Duration.days)
+        True
 
     Conversion to string uses the unit originally used for defining
     the quantity and the ``%g%s`` format::
@@ -872,39 +954,6 @@ class Duration(object):
         '1hour'
 
     """
-    __metaclass__ = Quantity(
-        # base unit is nanoseconds; use the SI symbol 'ns'
-        'ns',
-        # alternate spellings
-        nanosec=1,
-        nanoseconds=1,
-        # microsecond(s)
-        ms=1000,
-        microsec=1000,
-        microseconds=1000,
-        # seconds(s)
-        s=10**6,
-        sec=10**6,
-        secs=10**6,
-        second=10**6,
-        seconds=10**6,
-        # minute(s)
-        m=60 * 10**6,
-        min=60 * 10**6,
-        mins=60 * 10**6,
-        minute=60 * 10**6,
-        minutes=60 * 10**6,
-        # hour(s)
-        h=60*60 * 10**6,
-        hr=60*60 * 10**6,
-        hrs=60*60 * 10**6,
-        hour=60*60 * 10**6,
-        hours=60*60 * 10**6,
-        # day(s)
-        d=24*60*60 * 10**6,
-        day=24*60*60 * 10**6,
-        days=24*60*60 * 10**6,
-        )
 
     # override ctor to hook `_new_from_timedelta` in
     def __new__(cls, val, unit=None, name=None):
@@ -933,7 +982,7 @@ class Duration(object):
                     lapse += int(match.group(name)) * unit_lapse
                     last_unit = unit
             return cls._new_from_amount_and_unit(
-                amount=(lapse / last_unit.amount(Duration.s)),
+                amount=(old_div(lapse, last_unit.amount(Duration.s))),
                 unit=last_unit)
         elif ':' in val:
             # since `val` didn't match `_TIMESPEC_RE`, then it must
@@ -952,8 +1001,8 @@ class Duration(object):
                                 val1, val2, val1, val2,
                                 val1, val2, val1, val2))
         else:
-            # strings like `30 seconds` can be parsed by `_Quantity._new_from_string`;
-            # see http://blogs.gnome.org/jamesh/2005/06/23/overriding-class-methods-in-python/
+            # strings like `30 seconds` can be parsed by
+            # `_Quantity._new_from_string`; see http://goo.gl/pY44W5
             return super(Duration, cls)._new_from_string(val)
 
     @classmethod
@@ -979,7 +1028,8 @@ class Duration(object):
         """
         try:
             # Python 2.7 onwards
-            return cls._new_from_amount_and_unit(int(td.total_seconds()), Duration.s)
+            return cls._new_from_amount_and_unit(
+                int(td.total_seconds()), Duration.s)
         except AttributeError:
             return cls._new_from_amount_and_unit(
                 int(td.seconds + td.days * 24 * 3600),
@@ -998,8 +1048,12 @@ class Duration(object):
 # needed by `Duration._new_from_string`
 _TIMESPEC_RE = re.compile(
     r'('
-    # 1. allow HH:MM:SS or DD:HH:MM:SS; the form XX:YY is rejected because of ambiguity
-    r'((?P<days1>[0-9]+):)?(?P<hours1>[0-9]+):(?P<minutes1>[0-9]+):(?P<secs1>[0-9]+)'
+    # 1. allow HH:MM:SS or DD:HH:MM:SS; the form XX:YY is rejected because of
+    # ambiguity
+    r'((?P<days1>[0-9]+):)?'
+    '(?P<hours1>[0-9]+):'
+    '(?P<minutes1>[0-9]+):'
+    '(?P<secs1>[0-9]+)'
     r'|'
     # 2. allow `4days 3hrs`, `1 hour 4 minutes` and abbreviations `1d 2h 4m`
     r'((?P<days2>[0-9]+) \s* d(ays?))? \s*'
@@ -1008,24 +1062,24 @@ _TIMESPEC_RE = re.compile(
     r'((?P<secs2>[0-9]+) \s* s(ecs?))?'
     # 3. everything else is forwarded to `_Quantity._new_from_string`
     r')',
-    re.X|re.I)
+    re.X | re.I)
 _TIMESPEC_VALS = [
     # regexp group name
     # |          unit time lapse as seconds
     # |          |         corresponding `Duration` unit
     # |          |         |
-    ('days1',    24*60*60, Duration.day),
-    ('hours1',   60*60,    Duration.hour),
-    ('minutes1', 60,       Duration.minute),
-    ('secs1',    1,        Duration.s),
-    ('days2',    24*60*60, Duration.day),
-    ('hours2',   60*60,    Duration.hour),
-    ('minutes2', 60,       Duration.minute),
-    ('secs2',    1,        Duration.s),
-    ]
+    ('days1', 24 * 60 * 60, Duration.day),
+    ('hours1', 60 * 60, Duration.hour),
+    ('minutes1', 60, Duration.minute),
+    ('secs1', 1, Duration.s),
+    ('days2', 24 * 60 * 60, Duration.day),
+    ('hours2', 60 * 60, Duration.hour),
+    ('minutes2', 60, Duration.minute),
+    ('secs2', 1, Duration.s),
+]
 
 
-## aliases for common units
+# aliases for common units
 
 B = Memory.B
 byte = B
@@ -1057,7 +1111,7 @@ d = Duration.d
 days = d
 
 
-## main: run tests
+# main: run tests
 
 if "__main__" == __name__:
     import doctest
